from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):
    # Format: (code, name)
    currencies = (
        ("AED", "Dírham de los Emiratos Árabes Unidos"),
        ("AFN", "Afghaní"),
        ("ALL", "Lek albanés"),
        ("AMD", "Dram armenio"),
        ("ANG", "Florín de las Antillas Holandesas"),
        ("AOA", "Kwanza angoleño"),
        ("ARS", "Peso argentino"),
        ("AUD", "Dólar australiano"),
        ("AWG", "Florín arubeño"),
        ("AZN", "Manat azerbaiyano"),
        ("BAM", "Marco bosnioherzegovino"),
        ("BBD", "Dólar barbadense"),
        ("BDT", "Taka bangladesí"),
        ("BGN", "Lev búlgaro"),
        ("BHD", "Dinar bahreiní"),
        ("BIF", "Franco burundés"),
        ("BMD", "Dólar de Bermudas"),
        ("BND", "Dólar bruneano"),
        ("BOB", "Boliviano"),
        ("BRL", "Real brasileño"),
        ("BSD", "Dólar bahameño"),
        ("BTN", "Ngultrum butanés"),
        ("BWP", "Pula de Botswana"),
        ("BYR", "Rublio bielurruso"),
        ("BZD", "Dólar beliceño"),
        ("CAD", "Dólar canadiense"),
        ("CDF", "Franco congolés"),
        ("CHF", "Franco suizo"),
        ("CLP", "Peso chileno"),
        ("CNY", "Yuan"),
        ("COP", "Peso colombiano"),
        ("CRC", "Colón costarricense"),
        ("CUC", "Peso cubano convertible"),
        ("CUP", "Peso subano"),
        ("CVE", "Escudo de Cabo Verde"),
        ("CZK", "Corona checa"),
        ("DJF", "Franco yibutiano"),
        ("DKK", "Corona danesa"),
        ("DOP", "Peso dominicano"),
        ("DZD", "Dinar argelino"),
        ("EGP", "Libra egipcia"),
        ("ERN", "Nafka"),
        ("ETB", "Bir de Etiopía"),
        ("EUR", "Euro"),
        ("FJD", "Dólar fiyiano"),
        ("FKP", "Libra de las islas Falkland"),
        ("GBP", "Libra esterlina"),
        ("GEL", "Larí georgiano"),
        ("GGP", "Libra de Guernsey"),
        ("GHS", "Cedi"),
        ("GIP", "Libra de Gibraltar"),
        ("GMD", "Dalasi"),
        ("GNF", "Franco guineano"),
        ("GTQ", "Quetzal guatemalteco"),
        ("GYD", "Dólar guyanés"),
        ("HKD", "Dólar hongkonés"),
        ("HNL", "Lempira hondureño"),
        ("HRK", "Kuna croata"),
        ("HTG", "Gourde haitiano"),
        ("HUF", "Forinto húngaro"),
        ("IDR", "Rupia indonesia"),
        ("ILS", "Séquel israelí"),
        ("NIS", "Nuevo Séquel israelí"),
        ("IMP", "Libra manesa"),
        ("INR", "Rupia india"),
        ("IQD", "Dinar iraquí"),
        ("IRR", "Rial iraní"),
        ("ISK", "Corona islandesa"),
        ("JEP", "Libra de Jersey"),
        ("JMD", "Dólar jamaicano"),
        ("JOD", "Dinar jordano"),
        ("JPY", "Yen japonés"),
        ("KES", "Chelín keniano"),
        ("KGS", "Som kirguís"),
        ("KHR", "Riel camboyano"),
        ("KMF", "Franco comorense"),
        ("KPW", "Won norcoreano"),
        ("KRW", "Krahn Occidental"),
        ("KWD", "Dinar kuwaití"),
        ("KYD", "Dólar de las islas Cayman"),
        ("KZT", "Tenge kazako"),
        ("LAK", "Kip laosiano"),
        ("LBP", "Libra libanesa"),
        ("LKR", "Rupia esrilanquesa"),
        ("LRD", "Dólar liberiano"),
        ("LSL", "Loti lesothense"),
        ("LTL", "Litas lituana"),
        ("LYD", "Dinar libio"),
        ("MAD", "Dirham marroquí"),
        ("MDL", "Leu moldavo"),
        ("MGA", "Ariary malgache"),
        ("MKD", "Denar normacedonio"),
        ("MMK", "Kyat birmano"),
        ("MNT", "Tugrik mongol"),
        ("MOP", "Pataca macaense"),
        ("MRO", "Ouguiya mauritano"),
        ("MUR", "Rupia mauritana"),
        ("MVR", "Rupia de Maldivas"),
        ("MWK", "Kwacha malauí"),
        ("MXN", "Peso mexicano"),
        ("MYR", "Ringgit"),
        ("MZN", "Metical mozambiqueño"),
        ("NAD", "Dólar namibio"),
        ("NGN", "Naira nigeriano"),
        ("NIO", "Córdoba nicaragüense"),
        ("NOK", "Corona noruega"),
        ("NPR", "Rupia nepalí"),
        ("NZD", "Dólar neozelandés"),
        ("OMR", "Rial omaní"),
        ("PAB", "Balboa panameño"),
        ("PEN", "Sol peruano"),
        ("PGK", "Kina"),
        ("PHP", "Peso filipino"),
        ("PKR", "Rupia pakistaní"),
        ("PLN", "Złoty polaco"),
        ("PYG", "Guaraní paraguayo"),
        ("QAR", "Riyal catarí"),
        ("RON", "Leu rumano"),
        ("RSD", "Dinar serbio"),
        ("RUB", "Rublo ruso"),
        ("RWF", "Franco ruandés"),
        ("SAR", "Riyal saudí"),
        ("SBD", "Dólar de las islas Solomon"),
        ("SCR", "Rupia seychellense"),
        ("SDG", "Libra sudanesa"),
        ("SEK", "Corona sueca"),
        ("SGD", "Dólar de Singapur"),
        ("SHP", "Libra de Santa Elena"),
        ("SLL", "Leona"),
        ("SOS", "Chelín somalí"),
        ("SPL", "Luigino"),
        ("SRD", "Dólar surinamés"),
        ("STD", "Dobra santotomense"),
        ("SVC", "Colón salvadoreño"),
        ("SYP", "Libra siria"),
        ("SZL", "Lilangeni"),
        ("THB", "Baht tailandés"),
        ("TJS", "Somoni tayiko"),
        ("TMT", "Manat turcomano"),
        ("TND", "Dinar tunecino"),
        ("TOP", "Pa'anga tongano"),
        ("TRY", "Lira turca"),
        ("TTD", "Dólar de Trinidad and Tobago"),
        ("TVD", "Dólar tuvaluano"),
        ("TWD", "Nuevo dólar taiwanés"),
        ("TZS", "Chelín tanzano"),
        ("UAH", "Grivna ucraniano"),
        ("UGX", "Chelín ugandés"),
        ("USD", "Dólar de Estados Unidos"),
        ("UYU", "Peso uruguayo"),
        ("UZS", "Soʻm Uzbekistani"),
        ("VEF", "Bolívar venezolano"),
        ("VND", "Đồng vietnamita"),
        ("VUV", "Vanuatu vatu"),
        ("WST", "Tālā samoano"),
        ("XAF", "Franco centro africano"),
        ("XCD", "Dólar del Caribe Oriental"),
        ("XDR", "Derechos especiales de giro"),
        ("XOF", "Franco de África occidental"),
        ("XPF", "Franco CFP"),
        ("YER", "Rial yemení"),
        ("ZAR", "Rand sudafricano"),
        ("ZMW", "Kwacha zambiano"),
        ("ZWD", "Dólar zimbabuense"),
    )



import pytest

from faker import Faker
from faker.exceptions import UniquenessException


class TestUniquenessClass:
    def test_uniqueness(self):
        fake = Faker("en_US")

        names = set()
        # There are (at time of writing 690) first names in the
        # US identity provider. Birthday paradox puts the chances of
        # no duplicates in 250 selections as low enough to be impossible
        for i in range(250):
            first_name = fake.unique.first_name()
            assert first_name not in names
            names.add(first_name)

    def test_sanity_escape(self):
        fake = Faker()

        # Those of you who are especially astute may realise
        # there are only 2 booleans, so the third boolean cannot
        # be unique.
        with pytest.raises(UniquenessException, match=r"Got duplicated values after [\d,]+ iterations."):
            for i in range(3):
                _ = fake.unique.boolean()

    def test_uniqueness_clear(self):
        fake = Faker()

        for i in range(2):
            fake.unique.boolean()

        fake.unique.clear()

        # Because we cleared the generated values, this will not
        # throw an exception
        fake.unique.boolean()

    def test_exclusive_arguments(self):
        """Calls through the "unique" portal will only affect
        calls with that specific function signature.
        """
        fake = Faker()

        for i in range(10):
            fake.unique.random_int(min=1, max=10)

        # Different signature, so new pool. If they shared a pool
        # this would throw a sanity exception
        fake.unique.random_int(min=2, max=10)

    def test_functions_only(self):
        """Accessing non-functions through the `.unique` attribute
        will throw a TypeError."""

        fake = Faker()

        with pytest.raises(TypeError, match="Accessing non-functions through .unique is not supported."):
            fake.unique.locales



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``da_DK`` locale.
    Source: https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Denmark
    """

    license_formats = ("?? ## ###",)



import pytest

from faker.decode import unidecode


@pytest.mark.parametrize(
    ("text", "result"),
    [
        (
            "Programmes de publicité - Solutions d'entreprise",
            "Programmes de publicite - Solutions d'entreprise",
        ),
        ("Транслитерирует и русский", "Transliteriruet i russkii"),
        ("kožušček", "kozuscek"),
        ("北亰", "Bei Jing "),
        ("ĳ", "ij"),
    ],
)
def test_transliterate(text, result):
    assert unidecode(text) == result


@pytest.mark.parametrize("code", range(128))
def test_7bit_purity(code):
    ch = chr(code)
    assert unidecode(ch) == ch


def test_7bit_text_purity():
    txt = "".join([chr(x) for x in range(128)])
    assert unidecode(txt) == txt


def test_out_of_bounds():
    assert unidecode("𐀀") == ""



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    safe_email_tlds = ("com", "net", "fr", "fr")
    free_email_domains = (
        "voila.fr",
        "gmail.com",
        "hotmail.fr",
        "yahoo.fr",
        "laposte.net",
        "free.fr",
        "sfr.fr",
        "orange.fr",
        "bouygtel.fr",
        "club-internet.fr",
        "dbmail.com",
        "live.com",
        "ifrance.com",
        "noos.fr",
        "tele2.fr",
        "tiscali.fr",
        "wanadoo.fr",
    )
    tlds = ("com", "com", "com", "net", "org", "fr", "fr", "fr")
    replacements = (
        ("à", "a"),
        ("â", "a"),
        ("ä", "a"),
        ("ç", "c"),
        ("é", "e"),
        ("è", "e"),
        ("ê", "e"),
        ("ë", "e"),
        ("É", "e"),
        ("ï", "i"),
        ("î", "i"),
        ("ô", "o"),
        ("ö", "o"),
        ("ù", "u"),
        ("ü", "u"),
    )



from typing import Tuple

from .. import Provider as AddressProvider


class Provider(AddressProvider):
    city_suffixes = (
        "-des-Bois",
        "-les-Bains",
        "-la-Ville",
        "-Dessus",
        "-Dessous",
        " am Rhein",
        " am See",
        " am Albis",
        " an der Aare",
    )
    city_prefixes = ("Saint ", "Sainte ", "San ", "Ober", "Unter")
    street_prefixes = ("rue", "rue", "chemin", "avenue", "boulevard")

    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    building_number_formats = ("%", "%#", "%#", "%#", "%##")

    city_formats = (
        "{{last_name}}",
        "{{last_name}}",
        "{{last_name}}",
        "{{last_name}}",
        "{{last_name}}{{city_suffix}}",
        "{{last_name}}{{city_suffix}}",
        "{{last_name}}{{city_suffix}}",
        "{{last_name}}-près-{{last_name}}",
        "{{last_name}}-sur-{{last_name}}",
        "{{city_prefix}}{{last_name}}",
        "{{last_name}} ({{canton_code}})",
    )

    street_address_formats = (
        "{{street_name}}",
        "{{street_name}} {{building_number}}",
        "{{street_name}} {{building_number}}",
        "{{street_name}} {{building_number}}",
        "{{street_name}} {{building_number}}",
        "{{street_name}} {{building_number}}",
    )
    street_name_formats = (
        "{{street_prefix}} {{last_name}}",
        "{{street_prefix}} {{first_name}} {{last_name}}",
        "{{street_prefix}} de {{last_name}}",
    )

    postcode_formats = (
        "1###",
        "2###",
        "3###",
        "4###",
        "5###",
        "6###",
        "7###",
        "8###",
        "9###",
    )

    cantons = (
        ("AG", "Argovie"),
        ("AI", "Appenzell Rhodes-Intérieures"),
        ("AR", "Appenzell Rhodes-Extérieures"),
        ("BE", "Berne"),
        ("BL", "Bâle-Campagne"),
        ("BS", "Bâle-Ville"),
        ("FR", "Fribourg"),
        ("GE", "Genève"),
        ("GL", "Glaris"),
        ("GR", "Grisons"),
        ("JU", "Jura"),
        ("LU", "Lucerne"),
        ("NE", "Neuchâtel"),
        ("NW", "Nidwald"),
        ("OW", "Obwald"),
        ("SG", "Saint-Gall"),
        ("SH", "Schaffhouse"),
        ("SO", "Soleure"),
        ("SZ", "Schwytz"),
        ("TG", "Thurgovie"),
        ("TI", "Tessin"),
        ("UR", "Uri"),
        ("VD", "Vaud"),
        ("VS", "Valais"),
        ("ZG", "Zoug"),
        ("ZH", "Zurich"),
    )

    countries = (
        "Afghanistan",
        "Afrique du sud",
        "Albanie",
        "Algérie",
        "Allemagne",
        "Andorre",
        "Angola",
        "Anguilla",
        "Antarctique",
        "Antigua et Barbuda",
        "Antilles néerlandaises",
        "Arabie saoudite",
        "Argentine",
        "Arménie",
        "Aruba",
        "Australie",
        "Autriche",
        "Azerbaïdjan",
        "Bahamas",
        "Bahrain",
        "Bangladesh",
        "Belgique",
        "Belize",
        "Benin",
        "Bermudes (Les)",
        "Bhoutan",
        "Biélorussie",
        "Bolivie",
        "Bosnie-Herzégovine",
        "Botswana",
        "Bouvet (Îles)",
        "Brunei",
        "Brésil",
        "Bulgarie",
        "Burkina Faso",
        "Burundi",
        "Cambodge",
        "Cameroun",
        "Canada",
        "Cap Vert",
        "Cayman (Îles)",
        "Chili",
        "Chine (Rép. pop.)",
        "Christmas (Île)",
        "Chypre",
        "Cocos (Îles)",
        "Colombie",
        "Comores",
        "Cook (Îles)",
        "Corée du Nord",
        "Corée, Sud",
        "Costa Rica",
        "Croatie",
        "Cuba",
        "Côte d'Ivoire",
        "Danemark",
        "Djibouti",
        "Dominique",
        "Égypte",
        "El Salvador",
        "Émirats arabes unis",
        "Équateur",
        "Érythrée",
        "Espagne",
        "Estonie",
        "États-Unis",
        "Ethiopie",
        "Falkland (Île)",
        "Fidji (République des)",
        "Finlande",
        "France",
        "Féroé (Îles)",
        "Gabon",
        "Gambie",
        "Ghana",
        "Gibraltar",
        "Grenade",
        "Groenland",
        "Grèce",
        "Guadeloupe",
        "Guam",
        "Guatemala",
        "Guinée",
        "Guinée Equatoriale",
        "Guinée-Bissau",
        "Guyane",
        "Guyane française",
        "Géorgie",
        "Géorgie du Sud et Sandwich du Sud (Îles)",
        "Haïti",
        "Heard et McDonald (Îles)",
        "Honduras",
        "Hong Kong",
        "Hongrie",
        "Îles Mineures Éloignées des États-Unis",
        "Inde",
        "Indonésie",
        "Irak",
        "Iran",
        "Irlande",
        "Islande",
        "Israël",
        "Italie",
        "Jamaïque",
        "Japon",
        "Jordanie",
        "Kazakhstan",
        "Kenya",
        "Kirghizistan",
        "Kiribati",
        "Koweit",
        "La Barbad",
        "Laos",
        "Lesotho",
        "Lettonie",
        "Liban",
        "Libye",
        "Libéria",
        "Liechtenstein",
        "Lithuanie",
        "Luxembourg",
        "Macau",
        "Macédoine du Nord",
        "Madagascar",
        "Malaisie",
        "Malawi",
        "Maldives (Îles)",
        "Mali",
        "Malte",
        "Mariannes du Nord (Îles)",
        "Maroc",
        "Marshall (Îles)",
        "Martinique",
        "Maurice",
        "Mauritanie",
        "Mayotte",
        "Mexique",
        "Micronésie (États fédérés de)",
        "Moldavie",
        "Monaco",
        "Mongolie",
        "Montserrat",
        "Mozambique",
        "Myanmar",
        "Namibie",
        "Nauru",
        "Nepal",
        "Nicaragua",
        "Niger",
        "Nigeria",
        "Niue",
        "Norfolk (Îles)",
        "Norvège",
        "Nouvelle Calédonie",
        "Nouvelle-Zélande",
        "Oman",
        "Ouganda",
        "Ouzbékistan",
        "Pakistan",
        "Palau",
        "Panama",
        "Papouasie-Nouvelle-Guinée",
        "Paraguay",
        "Pays-Bas",
        "Philippines",
        "Pitcairn (Îles)",
        "Pologne",
        "Polynésie française",
        "Porto Rico",
        "Portugal",
        "Pérou",
        "Qatar",
        "Roumanie",
        "Royaume-Uni",
        "Russie",
        "Rwanda",
        "Rép. Dém. du Congo",
        "République centrafricaine",
        "République Dominicaine",
        "République tchèque",
        "Réunion (La)",
        "Sahara Occidental",
        "Saint Pierre et Miquelon",
        "Saint Vincent et les Grenadines",
        "Saint-Kitts et Nevis",
        "Saint-Marin (Rép. de)",
        "Sainte Hélène",
        "Sainte Lucie",
        "Samoa",
        "Samoa",
        "Seychelles",
        "Sierra Leone",
        "Singapour",
        "Slovaquie",
        "Slovénie",
        "Somalie",
        "Soudan",
        "Sri Lanka",
        "Suisse",
        "Suriname",
        "Suède",
        "Svalbard et Jan Mayen (Îles)",
        "Swaziland",
        "Syrie",
        "São Tomé et Príncipe (Rép.)",
        "Sénégal",
        "Tadjikistan",
        "Taiwan",
        "Tanzanie",
        "Tchad",
        "Territoire britannique de l'océan Indien",
        "Territoires français du sud",
        "Thailande",
        "Timor",
        "Togo",
        "Tokelau",
        "Tonga",
        "Trinité et Tobago",
        "Tunisie",
        "Turkménistan",
        "Turks et Caïques (Îles)",
        "Turquie",
        "Tuvalu",
        "Ukraine",
        "Uruguay",
        "Vanuatu",
        "Vatican (Etat du)",
        "Venezuela",
        "Vierges (Îles)",
        "Vierges britanniques (Îles)",
        "Vietnam",
        "Wallis et Futuna (Îles)",
        "Yemen",
        "Yougoslavie",
        "Zambie",
        "Zaïre",
        "Zimbabwe",
    )

    def street_prefix(self) -> str:
        """
        :example: 'rue'
        """
        return self.random_element(self.street_prefixes)

    def city_prefix(self) -> str:
        """
        :example: 'rue'
        """
        return self.random_element(self.city_prefixes)

    def canton(self) -> Tuple[str, str]:
        """
        Randomly returns a swiss canton ('Abbreviated' , 'Name').
        :example: ('VD' . 'Vaud')
        """
        return self.random_element(self.cantons)

    def administrative_unit(self) -> str:
        """
        Randomly returns a Swiss canton name.
        :example: 'Vaud'
        """
        return self.canton()[1]

    canton_name = administrative_unit

    def canton_code(self) -> str:
        """
        Randomly returns a Swiss canton code.
        :example: 'VD'
        """
        return self.canton()[0]



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}}",
    )

    # Company suffixes are from
    # https://cs.wikipedia.org/wiki/Obchodn%C3%AD_spole%C4%8Dnost
    company_suffixes = (
        "s.r.o.",
        "o.s.",
        "a.s.",
        "v.o.s.",
        "k.s.",
    )



import csv
import datetime
import io
import itertools
import json
import re
import tarfile
import unittest
import uuid
import xml
import zipfile

try:
    import PIL.Image
except ImportError:
    PIL = None

try:
    import xmltodict
except ImportError:
    xmltodict = None

from typing import Pattern
from unittest.mock import patch

import pytest

from faker import Faker, exceptions
from faker.contrib.pytest.plugin import DEFAULT_LOCALE, DEFAULT_SEED


@pytest.fixture(scope="class")
def _class_faker_with_foobar():
    _fake = Faker(locale=DEFAULT_LOCALE)
    _fake.add_provider(_FooBarProvider())
    _fake.set_arguments("argument_group", "param", "Baz")
    _fake.set_arguments("double", "multi", 2)
    return _fake


@pytest.fixture()
def faker_with_foobar(request):
    fake = request.getfixturevalue("_class_faker_with_foobar")
    seed = DEFAULT_SEED
    if "faker_seed" in request.fixturenames:
        seed = request.getfixturevalue("faker_seed")
    fake.seed_instance(seed=seed)
    return fake


class _FooBarProvider:
    def foo_bar(self, param: str = None) -> str:
        return "FooBar" + str(param) if param else "FooBar"

    def test_integer(self, multi=1) -> int:
        return 1 * multi

    def test_float(self, multi=1) -> float:
        return 1.1 * multi

    def test_date_time(self) -> datetime.datetime:
        return datetime.datetime(2022, 12, 22, 13, 42, 33, 123)


class TestMiscProvider:
    """Test miscellaneous provider methods"""

    num_samples = 10

    def test_uuid4_str(self, faker, num_samples):
        pattern: Pattern = re.compile(r"[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}")
        for _ in range(num_samples):
            assert pattern.fullmatch(faker.uuid4())

    def test_uuid4_int(self, faker, num_samples):
        for _ in range(num_samples):
            assert isinstance(faker.uuid4(cast_to=int), int)

    def test_uuid4_uuid_object(self, faker, num_samples):
        for _ in range(num_samples):
            uuid4 = faker.uuid4(cast_to=None)
            assert isinstance(uuid4, uuid.UUID)
            assert uuid4.version == 4

    def test_uuid4_seedability(self, faker, num_samples):
        for _ in range(num_samples):
            random_seed = faker.random_int()
            faker.seed_instance(random_seed)
            expected_uuids = [faker.uuid4() for _ in range(100)]
            faker.seed_instance(random_seed)
            new_uuids = [faker.uuid4() for _ in range(100)]
            assert new_uuids == expected_uuids

    def test_zip_invalid_file(self, faker):
        with pytest.raises(ValueError):
            faker.zip(num_files="1")

        with pytest.raises(ValueError):
            faker.zip(num_files=0)

        with pytest.raises(ValueError):
            faker.zip(min_file_size="1")

        with pytest.raises(ValueError):
            faker.zip(min_file_size=0)

        with pytest.raises(ValueError):
            faker.zip(uncompressed_size="1")

        with pytest.raises(ValueError):
            faker.zip(uncompressed_size=0)

    def test_zip_one_byte_undersized(self, faker, num_samples):
        for _ in range(num_samples):
            num_files = faker.random.randint(1, 100)
            min_file_size = faker.random.randint(1, 1024)
            uncompressed_size = num_files * min_file_size - 1

            # Will always fail because of bad size requirements
            with pytest.raises(AssertionError):
                faker.zip(
                    uncompressed_size=uncompressed_size,
                    num_files=num_files,
                    min_file_size=min_file_size,
                )

    def test_zip_exact_minimum_size(self, faker, num_samples):
        for _ in range(num_samples):
            num_files = faker.random.randint(1, 100)
            min_file_size = faker.random.randint(1, 1024)
            uncompressed_size = num_files * min_file_size

            zip_bytes = faker.zip(
                uncompressed_size=uncompressed_size,
                num_files=num_files,
                min_file_size=min_file_size,
            )
            zip_buffer = io.BytesIO(zip_bytes)
            with zipfile.ZipFile(zip_buffer, "r") as zip_handle:
                # Verify zip archive is good
                assert zip_handle.testzip() is None

                # Verify zip archive has the correct number of files
                infolist = zip_handle.infolist()
                assert len(infolist) == num_files

                # Every file's size will be the minimum specified
                total_size = 0
                for info in infolist:
                    assert info.file_size == min_file_size
                    total_size += info.file_size

                # The file sizes should sum up to the specified uncompressed size
                assert total_size == uncompressed_size

    def test_zip_over_minimum_size(self, faker, num_samples):
        for _ in range(num_samples):
            num_files = faker.random.randint(1, 100)
            min_file_size = faker.random.randint(1, 1024)
            expected_extra_bytes = faker.random.randint(1, 1024 * 1024)
            uncompressed_size = num_files * min_file_size + expected_extra_bytes

            zip_bytes = faker.zip(
                uncompressed_size=uncompressed_size,
                num_files=num_files,
                min_file_size=min_file_size,
            )
            zip_buffer = io.BytesIO(zip_bytes)
            with zipfile.ZipFile(zip_buffer, "r") as zip_handle:
                # Verify zip archive is good
                assert zip_handle.testzip() is None

                # Verify zip archive has the correct number of files
                infolist = zip_handle.infolist()
                assert len(infolist) == num_files

                # Every file's size will be at least the minimum specified
                extra_bytes = 0
                total_size = 0
                for info in infolist:
                    assert info.file_size >= min_file_size
                    total_size += info.file_size
                    if info.file_size > min_file_size:
                        extra_bytes += info.file_size - min_file_size

                # The file sizes should sum up to the specified uncompressed size
                # and the extra bytes counted must be equal to the one expected
                assert total_size == uncompressed_size
                assert extra_bytes == expected_extra_bytes

    def test_zip_compression_py3(self, faker):
        num_files = 10
        min_file_size = 512
        uncompressed_size = 50 * 1024
        compression_mapping = [
            ("deflate", zipfile.ZIP_DEFLATED),
            ("gzip", zipfile.ZIP_DEFLATED),
            ("gz", zipfile.ZIP_DEFLATED),
            ("bzip2", zipfile.ZIP_BZIP2),
            ("bz2", zipfile.ZIP_BZIP2),
            ("lzma", zipfile.ZIP_LZMA),
            ("xz", zipfile.ZIP_LZMA),
            (None, zipfile.ZIP_STORED),
        ]
        for compression, compress_type in compression_mapping:
            zip_bytes = faker.zip(
                uncompressed_size=uncompressed_size,
                num_files=num_files,
                min_file_size=min_file_size,
                compression=compression,
            )
            zip_buffer = io.BytesIO(zip_bytes)
            with zipfile.ZipFile(zip_buffer, "r") as zip_handle:
                # Verify zip archive is good
                assert zip_handle.testzip() is None

                # Verify compression type used
                for info in zip_handle.infolist():
                    assert info.compress_type == compress_type

    def test_tar_invalid_file(self, faker):
        with pytest.raises(ValueError):
            faker.tar(num_files="1")

        with pytest.raises(ValueError):
            faker.tar(num_files=0)

        with pytest.raises(ValueError):
            faker.tar(min_file_size="1")

        with pytest.raises(ValueError):
            faker.tar(min_file_size=0)

        with pytest.raises(ValueError):
            faker.tar(uncompressed_size="1")

        with pytest.raises(ValueError):
            faker.tar(uncompressed_size=0)

    def test_tar_one_byte_undersized(self, faker, num_samples):
        for _ in range(num_samples):
            num_files = faker.random.randint(1, 100)
            min_file_size = faker.random.randint(1, 1024)
            uncompressed_size = num_files * min_file_size - 1

            # Will always fail because of conflicting size requirements
            with pytest.raises(AssertionError):
                faker.tar(
                    uncompressed_size=uncompressed_size,
                    num_files=num_files,
                    min_file_size=min_file_size,
                )

    def test_tar_exact_minimum_size(self, faker, num_samples):
        for _ in range(num_samples):
            num_files = faker.random.randint(1, 100)
            min_file_size = faker.random.randint(1, 1024)
            uncompressed_size = num_files * min_file_size

            tar_bytes = faker.tar(
                uncompressed_size=uncompressed_size,
                num_files=num_files,
                min_file_size=min_file_size,
            )
            tar_buffer = io.BytesIO(tar_bytes)
            with tarfile.open(fileobj=tar_buffer) as tar_handle:
                # Verify tar has the correct number of files
                members = tar_handle.getmembers()
                assert len(members) == num_files

                # Every file's size will be the minimum specified
                total_size = 0
                for member in members:
                    assert member.size == min_file_size
                    total_size += member.size

                # The file sizes should sum up to the specified uncompressed size
                assert total_size == uncompressed_size

    def test_tar_over_minimum_size(self, faker, num_samples):
        for _ in range(num_samples):
            num_files = faker.random.randint(1, 100)
            min_file_size = faker.random.randint(1, 1024)
            expected_extra_bytes = faker.random.randint(1, 1024 * 1024)
            uncompressed_size = num_files * min_file_size + expected_extra_bytes

            tar_bytes = faker.tar(
                uncompressed_size=uncompressed_size,
                num_files=num_files,
                min_file_size=min_file_size,
            )
            tar_buffer = io.BytesIO(tar_bytes)
            with tarfile.open(fileobj=tar_buffer) as tar_handle:
                # Verify tar has the correct number of files
                members = tar_handle.getmembers()
                assert len(members) == num_files

                # Every file's size will be at least the minimum specified
                extra_bytes = 0
                total_size = 0
                for member in members:
                    assert member.size >= min_file_size
                    total_size += member.size
                    if member.size > min_file_size:
                        extra_bytes += member.size - min_file_size

                # The file sizes should sum up to the specified uncompressed size
                # and the extra bytes counted should be the one we expect
                assert total_size == uncompressed_size
                assert extra_bytes == expected_extra_bytes

    def test_tar_compression_py3(self, faker):
        num_files = 25
        min_file_size = 512
        uncompressed_size = 50 * 1024
        compression_mapping = [
            ("gzip", "r:gz"),
            ("gz", "r:gz"),
            ("bzip2", "r:bz2"),
            ("bz2", "r:bz2"),
            ("lzma", "r:xz"),
            ("xz", "r:xz"),
            (None, "r"),
        ]

        for compression, read_mode in compression_mapping:
            tar_bytes = faker.tar(
                uncompressed_size=uncompressed_size,
                num_files=num_files,
                min_file_size=min_file_size,
                compression=compression,
            )
            tar_buffer = io.BytesIO(tar_bytes)
            with tarfile.open(fileobj=tar_buffer, mode=read_mode) as tar_handle:
                # Verify tar has the correct number of files
                members = tar_handle.getmembers()
                assert len(members) == num_files

    @unittest.skipUnless(PIL, "requires the Python Image Library")
    def test_image(self, faker):
        img = PIL.Image.open(io.BytesIO(faker.image()))
        assert img.size == (256, 256)
        assert img.format == "PNG"
        img = PIL.Image.open(io.BytesIO(faker.image(size=(2, 2), image_format="tiff")))
        assert img.size == (2, 2)
        assert img.format == "TIFF"

    def test_image_no_pillow(self, faker):
        with patch.dict("sys.modules", {"PIL": None}):
            with pytest.raises(exceptions.UnsupportedFeature) as excinfo:
                faker.image()

            assert excinfo.value.name == "image"

    def test_dsv_with_invalid_values(self, faker):
        with pytest.raises(ValueError):
            faker.dsv(num_rows="1")

        with pytest.raises(ValueError):
            faker.dsv(num_rows=0)

        with pytest.raises(TypeError):
            faker.dsv(header=None, data_columns=1)

        with pytest.raises(TypeError):
            faker.dsv(header=1, data_columns=["???"])

        with pytest.raises(ValueError):
            faker.dsv(header=["Column 1", "Column 2"], data_columns=["???"])

    def test_dsv_no_header(self, faker, num_samples):
        data_columns = ["????", "?????"]
        for _ in range(num_samples):
            num_rows = faker.random.randint(1, 1000)
            dsv = faker.dsv(header=None, data_columns=data_columns, num_rows=num_rows)
            reader = csv.reader(io.StringIO(dsv), dialect="faker-csv")

            # Verify each row has correct number of columns
            for row in reader:
                assert len(row) == len(data_columns)

            # Verify correct number of lines read
            assert reader.line_num == num_rows

    def test_dsv_with_valid_header(self, faker, num_samples):
        header = ["Column 1", "Column 2"]
        data_columns = ["????", "?????"]
        for _ in range(num_samples):
            num_rows = faker.random.randint(1, 1000)
            dsv = faker.dsv(header=header, data_columns=data_columns, num_rows=num_rows)
            reader = csv.reader(io.StringIO(dsv), dialect="faker-csv")

            # Verify each row has correct number of columns
            for row in reader:
                assert len(row) == len(data_columns)

            # Verify correct number of lines read (additional header row)
            assert reader.line_num == num_rows + 1

    def test_dsv_with_row_ids(self, faker, num_samples):
        data_columns = ["????", "?????"]
        for _ in range(num_samples):
            counter = 0
            num_rows = faker.random.randint(1, 1000)
            dsv = faker.dsv(
                header=None,
                data_columns=data_columns,
                num_rows=num_rows,
                include_row_ids=True,
            )
            reader = csv.reader(io.StringIO(dsv), dialect="faker-csv")

            # Verify each row has correct number of columns
            # and row ids increment correctly
            for row in reader:
                assert len(row) == len(data_columns) + 1
                counter += 1
                assert row[0] == str(counter)

            # Verify correct number of lines read
            assert reader.line_num == num_rows

    def test_dsv_data_columns(self, faker):
        num_rows = 10
        data_columns = ["{{name}}", "#??-####", "{{address}}", "{{phone_number}}"]
        with patch.object(faker["en_US"].factories[0], "pystr_format") as mock_pystr_format:
            mock_pystr_format.assert_not_called()
            faker.dsv(data_columns=data_columns, num_rows=num_rows)

            # pystr_format will be called for each data column and each row
            calls = mock_pystr_format.call_args_list
            assert len(calls) == num_rows * len(data_columns)

            # Elements in data_columns will be used in sequence per row generation cycle
            column_cycle = itertools.cycle(data_columns)
            for args, kwargs in calls:
                assert args[0] == next(column_cycle)
                assert kwargs == {}

    def test_dsv_csvwriter_kwargs(self, faker):
        data_keys = ["header", "data_columns", "num_rows", "include_row_ids"]
        test_kwargs = {
            "dialect": "excel",
            "header": ["Column 1", "Column 2"],
            "data_columns": ["????", "?????"],
            "num_rows": 5,
            "include_row_ids": True,
            "delimiter": ";",
            "invalid_kwarg": "invalid_value",
        }
        with patch("faker.providers.misc.csv.writer") as mock_writer:
            mock_writer.assert_not_called()
            faker.dsv(**test_kwargs)
            assert mock_writer.call_count == 1

            # Remove all data generation kwargs
            for key in data_keys:
                del test_kwargs[key]

            # Verify csv.writer was called with the remaining kwargs
            for args, kwargs in mock_writer.call_args_list:
                assert kwargs == test_kwargs

    @unittest.skipUnless(xmltodict, "requires the Python xmltodict Library")
    def test_xml(self, faker):
        try:
            xml.etree.ElementTree.fromstring(faker.xml())
        except xml.etree.ElementTree.ParseError:
            raise AssertionError("The XML format is invalid.")

    def test_xml_no_xmltodict(self, faker):
        with patch.dict("sys.modules", {"xmltodict": None}):
            with pytest.raises(exceptions.UnsupportedFeature) as excinfo:
                faker.xml()

            assert excinfo.value.name == "xml"

    def test_csv_helper_method(self, faker):
        kwargs = {
            "header": ["Column 1", "Column 2"],
            "data_columns": ["????", "?????"],
            "num_rows": 5,
            "include_row_ids": True,
        }
        with patch("faker.providers.misc.Provider.dsv") as mock_dsv:
            mock_dsv.assert_not_called()
            faker.csv(**kwargs)
            kwargs["delimiter"] = ","
            mock_dsv.assert_called_once_with(**kwargs)

    def test_tsv_helper_method(self, faker):
        kwargs = {
            "header": ["Column 1", "Column 2"],
            "data_columns": ["????", "?????"],
            "num_rows": 5,
            "include_row_ids": True,
        }
        with patch("faker.providers.misc.Provider.dsv") as mock_dsv:
            mock_dsv.assert_not_called()
            faker.tsv(**kwargs)
            kwargs["delimiter"] = "\t"
            mock_dsv.assert_called_once_with(**kwargs)

    def test_psv_helper_method(self, faker):
        kwargs = {
            "header": ["Column 1", "Column 2"],
            "data_columns": ["????", "?????"],
            "num_rows": 5,
            "include_row_ids": True,
        }
        with patch("faker.providers.misc.Provider.dsv") as mock_dsv:
            mock_dsv.assert_not_called()
            faker.psv(**kwargs)
            kwargs["delimiter"] = "|"
            mock_dsv.assert_called_once_with(**kwargs)

    def test_json_with_arguments(self, faker_with_foobar):
        kwargs = {
            "data_columns": [
                ("item1", "{{ foo_bar:argument_group }}"),
                ("item2", "foo_bar", {"param": "BAZ"}),
            ],
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert json_data.get("item1") == "FooBarBaz"
        assert json_data.get("item2") == "FooBarBAZ"

    def test_json_multiple_rows(self, faker_with_foobar):
        kwargs = {
            "data_columns": {"item": "foo_bar"},
            "num_rows": 2,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert isinstance(json_data, list) and len(json_data) == 2

    def test_json_passthrough_values(self, faker_with_foobar):
        kwargs = {
            "data_columns": {
                "item1": 1,
                "item2": 1.0,
                "item3": True,
                "item4": "@fixed",
            },
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert json_data["item1"] == 1
        assert json_data["item2"] == 1.0
        assert json_data["item3"] is True
        assert json_data["item4"] == "fixed"

    def test_json_type_integrity_int(self, faker_with_foobar):
        kwargs = {
            "data_columns": {
                "item1": "test_integer",
                "item2": "test_integer:double",
            },
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))
        assert isinstance(json_data["item1"], int)
        assert json_data["item2"] == 2

    def test_json_type_integrity_float(self, faker_with_foobar):
        kwargs = {
            "data_columns": {
                "item1": "test_float",
                "item2": "test_float:double",
            },
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))
        assert isinstance(json_data["item1"], float)
        assert json_data["item2"] == 2.2

    def test_json_invalid_data_columns(self, faker_with_foobar):
        kwargs = {
            "data_columns": (("item", "foo_bar"),),
            "num_rows": 1,
        }
        with pytest.raises(TypeError) as excinfo:
            json.loads(faker_with_foobar.json(**kwargs))
        assert str(excinfo.value) == "Invalid data_columns type. Must be a dictionary or list"

    def test_json_list_format_invalid_arguments_type(self, faker_with_foobar):
        kwargs = {
            "data_columns": [("item", "foo_bar", ["wrong"])],
            "num_rows": 1,
        }
        with pytest.raises(TypeError) as excinfo:
            faker_with_foobar.json(**kwargs)
        assert str(excinfo.value) == "Invalid arguments type. Must be a dictionary"

    def test_json_list_format_nested_list_of_values(self, faker_with_foobar):
        kwargs = {
            "data_columns": [
                (
                    "list",
                    [
                        (None, "{{ foo_bar }}s"),
                        (None, "foo_bar"),
                    ],
                ),
            ],
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert json_data["list"][0] == "FooBars"
        assert json_data["list"][1] == "FooBar"

    def test_json_list_format_nested_list_of_objects(self, faker_with_foobar):
        kwargs = {
            "data_columns": [
                (
                    "list",
                    [
                        ("item", "{{ foo_bar }}s"),
                        ("item", "foo_bar"),
                    ],
                ),
            ],
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert json_data["list"][0]["item"] == "FooBars"
        assert json_data["list"][1]["item"] == "FooBar"

    def test_json_list_format_nested_objects(self, faker_with_foobar):
        kwargs = {
            "data_columns": [
                (
                    "dict",
                    (
                        ("item1", "{{ foo_bar }}s"),
                        ("item2", "foo_bar"),
                    ),
                ),
            ],
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert json_data["dict"]["item1"] == "FooBars"
        assert json_data["dict"]["item2"] == "FooBar"

    def test_json_dict_format_nested_list_of_values(self, faker_with_foobar):
        kwargs = {
            "data_columns": {
                "list": [
                    "{{ foo_bar }}s",
                    "foo_bar",
                ],
            },
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert json_data["list"][0] == "FooBars"
        assert json_data["list"][1] == "FooBar"

    def test_json_dict_format_nested_list_of_objects(self, faker_with_foobar):
        kwargs = {
            "data_columns": {
                "list": [
                    {"item": "{{ foo_bar }}s"},
                    {"item": "foo_bar"},
                ],
            },
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert json_data["list"][0]["item"] == "FooBars"
        assert json_data["list"][1]["item"] == "FooBar"

    def test_json_dict_format_nested_objects(self, faker_with_foobar):
        kwargs = {
            "data_columns": {
                "dict": {
                    "item1": "{{ foo_bar }}s",
                    "item2": "foo_bar",
                },
            },
            "num_rows": 1,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))

        assert json_data["dict"]["item1"] == "FooBars"
        assert json_data["dict"]["item2"] == "FooBar"

    def test_json_type_integrity_datetime_using_encoder(self, faker_with_foobar):
        class DateTimeEncoder(json.JSONEncoder):
            def default(self, obj):
                if isinstance(obj, datetime.datetime):
                    return str(obj)

                # Let the base class default method raise the TypeError
                return json.JSONEncoder.default(self, obj)

        kwargs = {
            "data_columns": {"item1": "test_date_time"},
            "num_rows": 1,
            "cls": DateTimeEncoder,
        }
        json_data = json.loads(faker_with_foobar.json(**kwargs))
        assert isinstance(json_data["item1"], str)
        assert json_data["item1"] == "2022-12-22 13:42:33.000123"

    def test_json_type_integrity_datetime_no_encoder(self, faker_with_foobar):
        kwargs = {"data_columns": {"item1": "test_date_time"}, "num_rows": 1}
        with pytest.raises(TypeError):
            faker_with_foobar.json(**kwargs)

    def test_json_bytes(self, faker_with_foobar):
        kwargs = {"data_columns": {"item1": "foo_bar"}, "num_rows": 1}
        json_data_bytes = faker_with_foobar.json_bytes(**kwargs)
        assert isinstance(json_data_bytes, bytes)
        json_data = json.loads(json_data_bytes.decode())
        assert json_data["item1"] == "FooBar"

    def test_fixed_width_with_arguments(self, faker_with_foobar):
        kwargs = {
            "data_columns": [
                (9, "{{ foo_bar:argument_group }}"),
                (9, "foo_bar", {"param": "BAR"}),
            ],
            "num_rows": 2,
        }
        fixed_width_string = faker_with_foobar.fixed_width(**kwargs)

        for row in fixed_width_string.split("\n"):
            assert len(row) == 18
            assert row[0:9].strip() == "FooBarBaz"
            assert row[9:18].strip() == "FooBarBAR"

    def test_fixed_width_invalid_arguments_type(self, faker_with_foobar):
        kwargs = {
            "data_columns": [(9, "foo_bar", ["wrong"])],
            "num_rows": 1,
        }
        with pytest.raises(TypeError) as excinfo:
            faker_with_foobar.fixed_width(**kwargs)
        assert str(excinfo.value) == "Invalid arguments type. Must be a dictionary"

    def test_md5(self, faker):
        assert isinstance(faker.md5(), str)
        assert isinstance(faker.md5(raw_output=True), bytes)

    def test_sha1(self, faker):
        assert isinstance(faker.sha1(), str)
        assert isinstance(faker.sha1(raw_output=True), bytes)

    def test_sha256(self, faker):
        assert isinstance(faker.sha256(), str)
        assert isinstance(faker.sha256(raw_output=True), bytes)



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    free_email_domains = (
        "telia.com",
        "gmail.com",
        "swipnet.se",
        "googlemail.com",
        "live.se",
        "spray.se",
        "yahoo.de",
    )
    tlds = ("com", "com", "com", "se", "se", "se", "net", "org")

    replacements = (
        ("å", "a"),
        ("Å", "A"),
        ("ä", "a"),
        ("Ä", "A"),
        ("ö", "o"),
        ("Ö", "O"),
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    """
    A Faker provider for generating fake Zulu names in South Africa.
    """

    formats = (
        "{{first_name_male}} {{last_name_male}}",
        "{{first_name_male}} {{last_name_male}}",
        "{{first_name_male}} {{last_name_male}}",
        "{{first_name_male}} {{last_name_male}}",
        "{{first_name_male}} {{last_name_male}}-{{last_name_male}}",
        "{{first_name_female}} {{last_name_female}}",
        "{{first_name_female}} {{last_name_female}}",
        "{{first_name_female}} {{last_name_female}}",
        "{{first_name_female}} {{last_name_female}}",
        "{{first_name_female}} {{last_name_female}}-{{last_name_female}}",
        "{{prefix_male}} {{first_name_male}} {{last_name_male}}",
        "{{prefix_female}} {{first_name_female}} {{last_name_female}}",
        "{{prefix_male}} {{first_name_male}} {{last_name_male}}",
        "{{prefix_female}} {{first_name_female}} {{last_name_female}}",
    )

    # first names sourced from:
    # 1. https://briefly.co.za/35929-230-zulu-names-boys-girls.html
    # 2. https://en.wiktionary.org/wiki/Appendix:Zulu_given_names
    first_names_male = (
        "Andile",
        "Bandile",
        "Bhekani",
        "Jabulani",
        "Langalibalele",
        "Lungile",
        "Luyanda",
        "Lwandile",
        "Mandla",
        "Mandlakhe",
        "Mcebisi",
        "Minenhle",
        "Mlungisi",
        "Mthokozisi",
        "Musa",
        "Mzamo",
        "Nhlanhla",
        "Njabulo",
        "S'fiso",
        "Sandile",
        "Sibonelo",
        "Sibusiso",
        "SimphiweyiNkosi",
        "Siyabonga",
        "Sphiwe",
        "Thabani",
        "Thalente",
        "Thando",
        "Vusumuzi",
        "Zenzele",
    )
    first_names_female = (
        "Amahle",
        "Bhekisisa",
        "Bhekizizwe",
        "Bhekokwakhe",
        "Busisiwe",
        "Dumisani",
        "Dumisile",
        "Gugu",
        "Gugulethu",
        "Khanyisile",
        "Londisizwe",
        "Londiwe",
        "Mnqobi",
        "Ndumiso",
        "Nkazimulo",
        "Nobantu",
        "Nobuhle",
        "Nokulunga",
        "Nomagugu",
        "Nomalanga",
        "Nomasonto",
        "Nomathalente",
        "Nomathemba",
        "Nomcebo",
        "Nomthandazo",
        "Nomusa",
        "Nomvula",
        "Nonhlanhla",
        "Nonjabulo",
        "Nothando",
        "Noxolo",
        "Nozibusiso",
        "Nozizwe",
        "Nqobizitha",
        "Olwethu",
        "Owethu",
        "Sibongile",
        "Sibusisiwe",
        "Sihawukele",
        "Silondile",
        "Simphiwe",
        "Siphesihle",
        "Sithembiso",
        "Thabisa",
        "Thabisile",
        "Thalente",
        "Thandazile",
        "Thandeka",
        "Thandiwe",
        "Thembeka",
        "Thembekile",
        "Thembile",
        "Thembisile",
        "Thuthukile",
        "Zakhele",
        "Zenzile",
        "Zibuyile",
        "Zinhle",
    )

    first_names = first_names_male + first_names_female

    # last names sourced from :
    # 1. https://briefly.co.za/27255-izithakazelo-zakwazulu-zulu-surnames-zulu-clan-names-list.html
    last_names_male = (
        "Amahle",
        "Andile",
        "Anele",
        "Ayanda",
        "Ayize",
        "Bantwini",
        "Bayeni",
        "Bhedleni",
        "Bele",
        "Bhebhe",
        "Bhejelibomvu",
        "Bhembe",
        "Bhengu",
        "Bhensela",
        "Bhovungana",
        "Bhungane",
        "Bikelwayo",
        "Bikwayo",
        "Binda",
        "Biyase",
        "Biyela",
        "Blose",
        "Bophela",
        "Boyabenyathi",
        "Bukhosini",
        "Caluza",
        "Cebisa",
        "Cebekhulu",
        "Cenge",
        "Chagwe",
        "Chamane",
        "Chibi",
        "Chibini",
        "Chiliza",
        "Cibane",
        "Chonco",
        "Cwalile",
        "Delwayo",
        "Dikane",
        "Dimba",
        "Dimbane",
        "Dimbani",
        "Dinabantu",
        "Dinangwe",
        "Dindela",
        "Dingila",
        "Dindi",
        "Dinwa",
        "Dlabane",
        "Dlabazane",
        "Dladla",
        "Dlakadla",
        "Dlakela",
        "Dlamane",
        "Dlambula",
        "Dlamdaka",
        "Dlamini",
        "Dlamlenze",
        "Dlangamandla",
        "Dlawuza",
        "Dlebenkomo",
        "Dlodlo",
        "Dlomo",
        "Dludla",
        "Dludlu",
        "Dlungwana",
        "Doncabe",
        "Donda",
        "Dubandlela",
        "Dubazane",
        "Dube",
        "Dukada",
        "Duma",
        "Dumakude",
        "Dumisa",
        "Dunge",
        "Duyaza",
        "Duze",
        "Dwala",
        "Fihlela",
        "Fakazi",
        "Fakude",
        "Fanisa",
        "Fenya",
        "Gabadela",
        "Gabela",
        "Gabhezi",
        "Gabhisa",
        "Gadlela",
        "Gazu",
        "Galu",
        "Gagashe",
        "Gama",
        "Gambu",
        "Gambuse",
        "Gamede",
        "Gasa",
        "Gasela",
        "Gatsheni",
        "Gcaba",
        "Gcaleka",
        "Gcugcwa",
        "Gcumisa",
        "Gcwabe",
        "Gcwensa",
        "Gebashe",
        "Gebhezi",
        "Gedeza",
        "Gence",
        "Gengeshe",
        "Gigaba",
        "Gina",
        "Gininda",
        "Goba",
        "Gobhozi",
        "Godide",
        "Goje",
        "Gotsholo",
        "Gubeshe",
        "Gubhela",
        "Gubhuza",
        "Gubulundu",
        "Gugushe",
        "Gule",
        "Guliwe",
        "Guma",
        "Gumede",
        "Gumbi",
        "Gwacela",
        "Gwagwa",
        "Gwala",
        "Gwamanda",
        "Gwanyana",
        "Gwija",
        "Gxabhashe",
        "Hadebe",
        "Hangala",
        "Hhoyiyane",
        "Hlabisa",
        "Hlabangane",
        "Hlatshwayo",
        "Hlela",
        "Hlengwa",
        "Hlomuka",
        "Hlongwa",
        "Hlongwane",
        "Hlophe",
        "Hlubi",
        "Hlumakazi",
        "Jali",
        "Jama",
        "Jamasijadu",
        "Jamile",
        "Jele",
        "Jibela",
        "Jili",
        "Jiyane",
        "Jobe",
        "Jokiwe",
        "Juqula",
        "Khaba",
        "Khabalidaka",
        "Khambule",
        "Khanyeza",
        "Khanyile",
        "Khaphela",
        "Khathi",
        "Khathini",
        "Khathide",
        "Khawula",
        "Kheswa",
        "Khezokhulu",
        "Kholose",
        "Khomo",
        "Khonjwayo",
        "Khosini",
        "Khoza",
        "Khuba",
        "Khubisa",
        "Khuboni",
        "Khukhuza",
        "Khulu",
        "Khuluse",
        "Khumalo",
        "Khumbuza",
        "Khuyameni",
        "Khuzwayo",
        "Khwela",
        "Khwane",
        "Kubheka",
        "Kunene",
        "Kweyama",
        "Lakaza",
        "Lamula",
        "Langa",
        "Lange",
        "Langeni",
        "Lembede",
        "Longode",
        "Lubelo",
        "Ludonga",
        "Lukhele",
        "Luqe",
        "Lutholoni",
        "Luthuli",
        "Luvuno",
        "Lwandle",
        "Mabanga",
        "Mabaso",
        "Mabhena",
        "Mabhoko",
        "Mabizela",
        "Mabika",
        "Mabhodla",
        "Mbutho",
        "Mabuya",
        "Mabuyakhulu",
        "Mabuza",
        "Macingwane",
        "Made",
        "Madela",
        "Madi",
        "Madiba",
        "Madide",
        "Madinane",
        "Madlala",
        "Madlanduna",
        "Madondo",
        "Madonsela",
        "Mdotshana",
        "Maduma",
        "Maduna",
        "Malembe",
        "Madlula",
        "Madziba",
        "Mafobo",
        "Mafulela",
        "Mafuleka",
        "Magagula",
        "Magubane",
        "Magutshwa",
        "Magwaza",
        "Mageba",
        "Mahaye",
        "Mahamba",
        "Mahlaba",
        "Mahlalela",
        "Mahlangu",
        "Mahlase",
        "Mahlinza",
        "Mahlobo",
        "Mahulube",
        "Majoka",
        "Majola",
        "Majozi",
        "Makhanya",
        "Makhaye",
        "Makhathini",
        "Makhaza",
        "Makhedama",
        "Makhoba",
        "Makhubo",
        "Makhulukhulu",
        "Makhunga",
        "Malambule",
        "Malevu",
        "Malinga",
        "Maluleka",
        "Mamba",
        "Manana",
        "Mangcamane",
        "Mantshinga",
        "Mangede",
        "Mangena",
        "Mangethe",
        "Manyoni",
        "Mnqamu",
        "Manqele",
        "Mnquhe",
        "Manzi",
        "Manzini",
        "Manzezulu",
        "Maphalala",
        "Maphindela",
        "Maphanga",
        "Maphisa",
        "Mapholoba",
        "Maphumulo",
        "Masango",
        "Maseko",
        "Mashaba",
        "Mashasha",
        "Mashinini",
        "Mashimane",
        "Mashiya",
        "Masibekela",
        "Masikane",
        "Masina",
        "Masinga",
        "Masilela",
        "Masondo",
        "Masuku",
        "Mathaba",
        "Mathebela",
        "Mathebula",
        "Mathebula",
        "Mathenjwa",
        "Mathetha",
        "Mathula",
        "Mathunjwa",
        "Mathibela",
        "Mathonsi",
        "Mathwasa",
        "Mavela",
        "Mavundla",
        "Mavuso",
        "Mawanda",
        "Mawewe",
        "Mayeza",
        "Mayise",
        "Mayisela",
        "Mazalankosi",
        "Mazibuko",
        "Mazwi",
        "Mbamali",
        "Magoza",
        "Magolwana",
        "Magononde",
        "Maziya",
        "Mbanjwa",
        "Mbatha",
        "Mbatshazwa",
        "Magujwa",
        "Mbembe",
        "Mbekwa",
        "Mbende",
        "Mbeje",
        "Mbhele",
        "Mbhense",
        "Mbhodwe",
        "Mbhobho",
        "Mbhulangwe",
        "Mbili",
        "Mbokazi",
        "Mboko",
        "Mbonambi",
        "Mbongwa",
        "Mbongwe",
        "Mbotho",
        "Mbonane",
        "Mboyisa",
        "Mbulazi",
        "Mbulaze",
        "Mbuli",
        "Mbungela",
        "Mbunjwa",
        "Mbuyazi",
        "Mbuyisa",
        "Mbuyise",
        "Mcambe",
        "Mcambi",
        "Mcanco",
        "Mcanyana",
        "Mchunu",
        "Mcoyi",
        "Mcwaye",
        "Mcusi",
        "Mdaka",
        "Mdakane",
        "Mdladla",
        "Mdlalose",
        "Mdlanyoka",
        "Mdletshe",
        "Mdlenevu",
        "Mdluli",
        "Mdlumbi",
        "Mdonswa",
        "Mdunge",
        "Mehloluhlaza",
        "Memela",
        "Nenzi",
        "Menziwa",
        "Meyiwa",
        "Mfeka",
        "Mfusi",
        "Mgabhi",
        "Mgabadeli",
        "Mgasela",
        "Mgazi",
        "Mgcaleka",
        "Mgenge",
        "Mgobhozi",
        "Mgilija",
        "Mhayise",
        "Mhlabandlovu",
        "Mhlambo",
        "Mhlanga",
        "Mhlangu",
        "Mhlanya",
        "Mhlongo",
        "Mhlongwane",
        "Mhlophe",
        "Mhlungu",
        "Mgwaba",
        "Miya",
        "Mjadu",
        "Mjoli",
        "Mjwara",
        "Mkhabela",
        "Mkhandlela",
        "Mkhathini",
        "Mkhatshwa",
        "Mkhithi",
        "Mkhize",
        "Mkhokeleleki",
        "Mkholo",
        "Mkhumbuzi",
        "Mkhungo",
        "Mkhonza",
        "Mkhwananzi",
        "Mlaba",
        "Mlalane",
        "Mlalazi",
        "Mlambo",
        "Mlangatshe",
        "Mlangeni",
        "Mlawula",
        "Mlungisi",
        "Mlungwana",
        "Mlondo",
        "Mlotshwa",
        "Mncube",
        "Mncwabe",
        "Mncwanga",
        "Mngadi",
        "Mngoma",
        "Mngomezulu",
        "Mngwemkhulu",
        "Mngwengwe",
        "Mnguni",
        "Mnikathi",
        "Mnisi",
        "Mnomiya",
        "Mnqayi",
        "Mqungebe",
        "Mnyandu",
        "Mnyoni",
        "Mondise",
        "Motha",
        "Mnangwe",
        "Mntambo",
        "Mntimande",
        "Mtolo",
        "Mntungwa",
        "Mnyamande",
        "Mpangazitha",
        "Mpanza",
        "Mphankomo",
        "Mphahlwa",
        "Mphazima",
        "Mphephethwa",
        "Mphemba",
        "Mpila",
        "Mpikela",
        "Mpofana",
        "Mpungose",
        "Mpumuza",
        "Mpunzana",
        "Mqadi",
        "Msane",
        "Msani",
        "Msamkhulu",
        "Mshazi",
        "Mshengu",
        "Mshibe",
        "Msibi",
        "Mshikela",
        "Mshikila",
        "Mshiyane",
        "Mseleku",
        "Msimang",
        "Msindazwe",
        "Msokazi",
        "Msomi",
        "Msweli",
        "Msuthu",
        "Mthabela",
        "Mthalane",
        "Mthanti",
        "Mthembu",
        "Mthethwa",
        "Mtimande",
        "Mthimkhulu",
        "Mthinti",
        "Mthiya",
        "Mthiyane",
        "Mthombeni",
        "Mthonti",
        "Mtshali",
        "Mtumaseli",
        "Mtuswa",
        "Musi",
        "Mvelase",
        "Mveni",
        "Mvubu",
        "Mvuyana",
        "Myeza",
        "Mwandla",
        "Mwelase",
        "Myeni",
        "Mzila",
        "Mzileni",
        "Mzilankatha",
        "Mzimela",
        "Mzizi",
        "Mzobe",
        "Mzolo",
        "Mzomba",
        "Mzoneli",
        "Mzukase",
        "Mzulwini",
        "Nala",
        "Nandisa",
        "Ncala",
        "Ncwaba",
        "Ncwane",
        "Ndandali",
        "Ntenga",
        "Ncama",
        "Ngcamane",
        "Ncanana",
        "Ncongwane",
        "Ncube",
        "Ncusi",
        "Ndaba",
        "Ndabandaba",
        "Ndabase",
        "Ndabansele",
        "Ndawo",
        "Njomane",
        "Nkundlande",
        "Ndabezitha",
        "Ndawonde",
        "Ndima",
        "Ndimande",
        "Ndinisa",
        "Ndiyema",
        "Ndlala",
        "Ndlandla",
        "Ndlangamandla",
        "Ndlanzi",
        "Ndlanya",
        "Ndlela",
        "Ndlondlo",
        "Ndosi",
        "Ndlovu",
        "Ndondakusuka",
        "Ndonga",
        "Nduli",
        "Ndwandwe",
        "Nene",
        "Ngazitha",
        "Ngcamu",
        "Ngcemu",
        "Ngcobo",
        "Ngcolosi",
        "Ngema",
        "Ngiba",
        "Zikhungwini",
        "Ngidi",
        "Ngobese",
        "Ngobisi",
        "Ngomane",
        "Ngonini",
        "Ngotsha",
        "Ngubane",
        "Ngubeni",
        "Ngungunyana",
        "Ngwane",
        "Ngwazi",
        "Ngwenya",
        "Ngwekazi",
        "Nhlabathi",
        "Nhlanhla",
        "Nhlanhlampofu",
        "Nhlane",
        "Nhlapho",
        "Nhleko",
        "Nhlengethwa",
        "Nkala",
        "Nkamzwayo",
        "Njiki",
        "Njinji",
        "Nkabinde",
        "Nkonyeni",
        "Nkosi",
        "Nkumane",
        "Nkomo",
        "Nkomose",
        "Nkomoye",
        "Nkwakha",
        "Nkwali",
        "Nkwaliyenkosi",
        "Nkwanyana",
        "Njapha",
        "Nodanga",
        "Nodlomo",
        "Nogantshi",
        "Nombela",
        "Nombhoco",
        "Nomndayi",
        "Nomvuma",
        "Nondaba",
        "Nondlela",
        "Nonduma",
        "Nongalaza",
        "Nongalo",
        "Nonkosi",
        "Nonkululeko",
        "Nontanda",
        "Nontuli",
        "Nonyana",
        "Nowanqa",
        "Nozulu",
        "Nqumela",
        "Nsele",
        "Nsibande",
        "Nsibanyoni",
        "Nsindane",
        "Nsukuza",
        "Ntaka",
        "Ntanzi",
        "Ntenga",
        "Ntsele",
        "Ntamonde",
        "Ntshangase",
        "Ntshalintshali",
        "Ntshingila",
        "Ntshiza",
        "Ntombela",
        "Ntuli",
        "Nxamalala",
        "Nxasana",
        "Nxele",
        "Nxumalo",
        "Nyazitla",
        "Nyambose",
        "Nyanda",
        "Nyandeni",
        "Nyathi",
        "Nyawo",
        "Nyawokhulu",
        "Nyawose",
        "Nyembe",
        "Nyembezi",
        "Nyide",
        "Nyokayebululu",
        "Nyongwana",
        "Nzama",
        "Nzamela",
        "Nzima",
        "Nzimande",
        "Nzimase",
        "Nzuza",
        "Onjengenyamazane",
        "Phakathi",
        "Phakathwayo",
        "Phathwayo",
        "Phetha",
        "Phethela",
        "Phewa",
        "Phikela",
        "Phingoshe",
        "Phoseka",
        "Phoswa",
        "Phungula",
        "Phuthini",
        "Qomazitha",
        "Qwabe",
        "Sabela",
        "Sabelo",
        "Sameya",
        "Sangwani",
        "Sangweni",
        "Sembathwa",
        "Sengwayo",
        "Shabane",
        "Shabane",
        "Shabalala",
        "Shabangu",
        "Shamase",
        "Shandu",
        "Shange",
        "Shangase",
        "Shazi",
        "Shengele",
        "Shelembe",
        "Shezi",
        "Dlaba",
        "Shibase",
        "Shinga",
        "Shoba",
        "Shombela",
        "Shoyisa",
        "Shozi",
        "Sibanda",
        "Sibalukhulu",
        "Sibaya",
        "Sibeko",
        "Sibisi",
        "Sibiya",
        "Sigagu",
        "Sigegede",
        "Sibhene",
        "Sibindi",
        "Sijadu",
        "Sikhakhane",
        "Sikhosana",
        "Sikobi",
        "Sikhunyana",
        "Sikwayo",
        "Silangwe",
        "Simelane",
        "Sithenjwa",
        "Sithole",
        "Sithombo",
        "Sithuli",
        "Sitolotolo",
        "Sishangwe",
        "Sishi",
        "Sishiya",
        "Siwele",
        "Siyaya",
        "Siyeshe",
        "Soduba",
        "Sokhela",
        "Sokhulu",
        "Sokhwebula",
        "Somboni",
        "Somfula",
        "Sompisi",
        "Sandanezwe",
        "Sondini",
        "Sondisa",
        "Sontuli",
        "Songiya",
        "Sonqandile",
        "Sothole",
        "Sotobe",
        "Swazi",
        "Tiba",
        "Thabethe",
        "Thabizolo",
        "Thango",
        "Thabekhulu",
        "Thela",
        "Thembekwayo",
        "Thembela",
        "Thenjwayo",
        "Thobeni",
        "Thole",
        "Thoyana",
        "Tukane",
        "Thuliswayo",
        "Thumbela",
        "Thumbeza",
        "Thusi",
        "Thusini",
        "Thwala",
        "Wanda",
        "Wasendlunkulu",
        "Wela",
        "Weza",
        "Vilakazi",
        "Vabaza",
        "Vangisa",
        "Vezi",
        "Vumisa",
        "Vundla",
        "Vunisa",
        "Xaba",
        "Xala",
        "Xhakaza",
        "Ximba",
        "Xolo",
        "Xulu",
        "Yengwa",
        "Yengwayo",
        "Yei",
        "Yeyeye",
    )

    # last names are not sex dependant
    last_names_female = last_names_male
    last_names = last_names_male + last_names_female

    prefixes_female = (
        "Mrs.",
        "Ms.",
        "Miss",
        "Dr.",
        "Nkosikazi",
        "Nkosazana",
        "Mama",
        "Dade",
    )

    prefixes_male = ("Mr.", "Dr.", "Mnumzane", "Baba", "Bhuti")



from unittest.mock import patch

import pytest

from faker import Faker, Generator


class BarProvider:
    def foo_formatter(self):
        return "barfoo"


class FooProvider:
    def foo_formatter(self):
        return "foobar"

    def foo_formatter_with_arguments(self, param="", append=""):
        return "baz" + str(param) + str(append)


@pytest.fixture(autouse=True)
def generator():
    generator = Generator()
    generator.add_provider(FooProvider())
    return generator


class TestGenerator:
    """Test Generator class"""

    def test_get_formatter_returns_correct_formatter(self, generator):
        foo_provider = generator.providers[0]
        formatter = generator.get_formatter("foo_formatter")
        assert callable(formatter) and formatter == foo_provider.foo_formatter

    def test_get_formatter_with_unknown_formatter(self, generator):
        with pytest.raises(AttributeError) as excinfo:
            generator.get_formatter("barFormatter")
        assert str(excinfo.value) == "Unknown formatter 'barFormatter'"

        fake = Faker("it_IT")
        with pytest.raises(AttributeError) as excinfo:
            fake.get_formatter("barFormatter")
        assert str(excinfo.value) == "Unknown formatter 'barFormatter' with locale 'it_IT'"

    def test_format_calls_formatter_on_provider(self, generator):
        assert generator.format("foo_formatter") == "foobar"

    def test_format_passes_arguments_to_formatter(self, generator):
        result = generator.format("foo_formatter_with_arguments", "foo", append="!")
        assert result == "bazfoo!"

    def test_add_provider_overrides_old_provider(self, generator):
        assert generator.format("foo_formatter") == "foobar"
        generator.add_provider(BarProvider())
        assert generator.format("foo_formatter") == "barfoo"

    def test_parse_without_formatter_tokens(self, generator):
        assert generator.parse("fooBar#?") == "fooBar#?"

    def test_parse_with_valid_formatter_tokens(self, generator):
        result = generator.parse('This is {{foo_formatter}} a text with "{{ foo_formatter }}"')
        assert result == 'This is foobar a text with "foobar"'

    def test_arguments_group_with_values(self, generator):
        generator.set_arguments("group1", "argument1", 1)
        generator.set_arguments("group1", "argument2", 2)
        assert generator.get_arguments("group1", "argument1") == 1
        assert generator.del_arguments("group1", "argument2") == 2
        assert generator.get_arguments("group1", "argument2") is None
        assert generator.get_arguments("group1") == {"argument1": 1}

    def test_arguments_group_with_dictionaries(self, generator):
        generator.set_arguments("group2", {"argument1": 3, "argument2": 4})
        assert generator.get_arguments("group2") == {"argument1": 3, "argument2": 4}
        assert generator.del_arguments("group2") == {"argument1": 3, "argument2": 4}
        assert generator.get_arguments("group2") is None

    def test_arguments_group_with_invalid_name(self, generator):
        assert generator.get_arguments("group3") is None
        assert generator.del_arguments("group3") is None

    def test_arguments_group_with_invalid_argument_type(self, generator):
        with pytest.raises(ValueError) as excinfo:
            generator.set_arguments("group", ["foo", "bar"])
        assert str(excinfo.value) == "Arguments must be either a string or dictionary"

    def test_parse_with_valid_formatter_arguments(self, generator):
        generator.set_arguments("format_name", {"param": "foo", "append": "bar"})
        result = generator.parse('This is "{{foo_formatter_with_arguments:format_name}}"')
        generator.del_arguments("format_name")
        assert result == 'This is "bazfoobar"'

    def test_parse_with_unknown_arguments_group(self, generator):
        with pytest.raises(AttributeError) as excinfo:
            generator.parse('This is "{{foo_formatter_with_arguments:unknown}}"')
        assert str(excinfo.value) == "Unknown argument group 'unknown'"

    def test_parse_with_unknown_formatter_token(self, generator):
        with pytest.raises(AttributeError) as excinfo:
            generator.parse("{{barFormatter}}")
        assert str(excinfo.value) == "Unknown formatter 'barFormatter'"

    def test_magic_call_calls_format(self, generator):
        assert generator.foo_formatter() == "foobar"

    def test_magic_call_calls_format_with_arguments(self, generator):
        assert generator.foo_formatter_with_arguments("foo") == "bazfoo"

    @patch("faker.generator.random_module.getstate")
    def test_get_random(self, mock_system_random, generator):
        random_instance = generator.random
        random_instance.getstate()
        mock_system_random.assert_not_called()

    @patch("faker.generator.random_module.seed")
    def test_random_seed_doesnt_seed_system_random(self, mock_system_random, generator):
        # Save original state of shared random instance to avoid affecting other tests
        state = generator.random.getstate()

        generator.seed(0)
        mock_system_random.assert_not_called()

        # Restore state of shared random instance
        generator.random.setstate(state)



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    free_email_domains = (
        "gmail.com",
        "daum.net",
        "hotmail.com",
        "hanmail.net",
        "naver.com",
        "nate.com",
        "live.com",
        "dreamwiz.com",
    )
    tlds = ("com", "com", "com", "kr", "kr", "net", "org")



from typing import Dict

from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``el_GR`` locale."""

    common_words = (
        "άρα",
        "ένα",
        "ένας",
        "έξι",
        "έτσι",
        "έχω",
        "ήδη",
        "ίδιο",
        "αν",
        "ανά",
        "από",
        "ας",
        "για",
        "δε",
        "δεν",
        "δύο",
        "εγώ",
        "εδώ",
        "εκτός",
        "επί",
        "θα",
        "κάτι",
        "και",
        "κι",
        "μέχρι",
        "μα",
        "μας",
        "με",
        "μη",
        "μην",
        "μια",
        "μιας",
        "μου",
        "να",
        "ναι",
        "ο",
        "οι",
        "πάντα",
        "πάντως",
        "πιο",
        "πλέον",
        "ποια",
        "πολύ",
        "που",
        "πως",
        "σαν",
        "σας",
        "σε",
        "σου",
        "στα",
        "στη",
        "στις",
        "στο",
        "τα",
        "τη",
        "την",
        "της",
        "τι",
        "τις",
        "το",
        "τον",
        "του",
        "τους",
        "των",
        "ως",
        "όσο",
        "όταν",
        "ότι",
        "όχι",
    )

    word_list = common_words * 2 + (
        "άλγεβρα",
        "άπειρα",
        "άρα",
        "άρθρων",
        "άτομο",
        "έγραψες",
        "έλεγχος",
        "έξι",
        "έρθει",
        "έστειλε",
        "έστελνε",
        "έτοιμος",
        "έτσι",
        "έχω",
        "ήδη",
        "ίδιο",
        "αγοράζοντας",
        "αθόρυβες",
        "ακούσει",
        "αλγόριθμου",
        "αναγκάζονται",
        "ανακλύψεις",
        "αναφέρονται",
        "αναφορά",
        "ανεπιθύμητη",
        "ανταγωνιστής",
        "αντιλήφθηκαν",
        "ανώδυνη",
        "απίστευτα",
        "απαράδεκτη",
        "απαραίτητο",
        "απαρατήρητο",
        "απλό",
        "αποδείξεις",
        "αποθηκευτικού",
        "αποκλειστικούς",
        "απομόνωση",
        "αποστηθίσει",
        "αποφάσισε",
        "από",
        "απόλαυσε",
        "αρέσει",
        "αρπάζεις",
        "αρχεία",
        "ατόμου",
        "αυτήν",
        "αυτός",
        "αφήσεις",
        "βάζοντας",
        "βαθμό",
        "βασανίζουν",
        "βγήκε",
        "βιαστικά",
        "βιβλίο",
        "βουτήξουν",
        "βρίσκονται",
        "γέλασαν",
        "γεγονός",
        "γειτονιάς",
        "γεύματος",
        "για",
        "γιαυτό",
        "γνωρίζουμε",
        "γνωστή",
        "γράψει",
        "γραμμές",
        "γραμμή",
        "γραμμής",
        "γραφικά",
        "δίνοντας",
        "δε",
        "δείξει",
        "δεδομένων",
        "δεν",
        "δημιουργήσεις",
        "δημιουργείς",
        "δημιουργια",
        "διάβασε",
        "διάσημα",
        "διαδίκτυο",
        "διακοπή",
        "διακοπής",
        "διακόψουμε",
        "διαπιστώνεις",
        "διασφαλίζεται",
        "διαφήμιση",
        "διαχειριστής",
        "διευθυντές",
        "διοικητικό",
        "διολισθήσεις",
        "διορθώσει",
        "διορθώσεις",
        "δοκιμάσεις",
        "δουλεύει",
        "δούλευε",
        "δυστυχής",
        "δυστυχώς",
        "δωροδοκηθούν",
        "δύο",
        "είχαμε",
        "εγώ",
        "εδώ",
        "ειδικά",
        "εικόνες",
        "εκδόσεις",
        "εκείνου",
        "εκθέσεις",
        "εκτελέσει",
        "εκτελέσεις",
        "εκτελείται",
        "εκτός",
        "ελέγχου",
        "εντολές",
        "εξακολουθεί",
        "εξαρτάται",
        "εξοργιστικά",
        "επί",
        "επενδυτής",
        "επεξεργασία",
        "επιδιορθώσεις",
        "επιδιόρθωση",
        "επιστρέφουν",
        "επιχείριση",
        "εργάστηκε",
        "εργαζόμενοι",
        "εργαζόμενων",
        "εργαλείων",
        "εργασίας",
        "εργοστασίου",
        "ερωτήσεις",
        "ερώτηση",
        "εσωτερικών",
        "εταιρείες",
        "ευκολότερο",
        "εφαμοργής",
        "εφαρμογή",
        "εφαρμογής",
        "ζητήσεις",
        "ημέρα",
        "θέλεις",
        "θέμα",
        "θέματα",
        "θυμάμαι",
        "ιδιαίτερα",
        "κάνε",
        "κάνεις",
        "κάτι",
        "και",
        "καλύτερο",
        "κανένας",
        "κανείς",
        "κανόνα",
        "καταλάθος",
        "κειμένων",
        "κι",
        "κλπ",
        "κοιτάζοντας",
        "κρατάει",
        "κρατήσουν",
        "κόλπα",
        "κόψεις",
        "κύκλο",
        "κώδικάς",
        "κώδικα",
        "λέει",
        "λίγο",
        "λαμβάνουν",
        "λες",
        "λετπά",
        "λιγότερο",
        "λοιπόν",
        "μάθε",
        "μάλλον",
        "μάτσο",
        "μέγιστη",
        "μέρος",
        "μέσης",
        "μέχρι",
        "μαγικά",
        "μερικούς",
        "μεταγλωτίσει",
        "μεταγλωτιστής",
        "μεταφραστής",
        "μετράει",
        "μετρήσεις",
        "μηχανής",
        "μπορούσες",
        "μπουν",
        "νέα",
        "νέο",
        "νέου",
        "νέων",
        "νιρβάνα",
        "νόμιζες",
        "ξέχασε",
        "ορίστε",
        "πάντα",
        "πάντως",
        "πάρα",
        "πάρεις",
        "πήρε",
        "παίξουν",
        "παίρνει",
        "παίρνουν",
        "πακέτων",
        "παράγοντες",
        "παράδειγμα",
        "παραγωγικής",
        "παραδοτέου",
        "παραδώσεις",
        "παραπάνω",
        "πεδία",
        "περίπου",
        "περιβάλλον",
        "περιβάλλοντος",
        "περιεχόμενα",
        "περιμένουν",
        "περισσότερες",
        "περισσότερη",
        "πες",
        "πετάνε",
        "πετάξαμε",
        "πετούν",
        "πηγαίου",
        "πιο",
        "πλέον",
        "ποια",
        "πολύ",
        "ποσοστό",
        "που",
        "προβληματική",
        "προγραμματιστές",
        "προγραμματιστής",
        "προκαλείς",
        "προκύπτουν",
        "προσεκτικά",
        "προσθέσει",
        "προσλάμβανες",
        "προσοχή",
        "προσπαθήσεις",
        "προσπαθούν",
        "προϊόντα",
        "πρόσληψη",
        "πρώτης",
        "πρώτο",
        "πρώτοι",
        "πόρτες",
        "ροή",
        "ρουτίνα",
        "ρωτάει",
        "ρωτήσει",
        "σίγουρος",
        "σημαντικό",
        "σημαντικός",
        "σημεία",
        "σκεφτείς",
        "σπίτι",
        "στέλνοντάς",
        "στήλες",
        "σταματάς",
        "στραβά",
        "συγγραφής",
        "συγγραφείς",
        "συγκεντρωμένοι",
        "συγχρόνως",
        "συγχωνευτεί",
        "συνάδελφος",
        "συνέχεια",
        "συνεντεύξεις",
        "συνεχώς",
        "συνηθίζουν",
        "σχεδιαστής",
        "σωστά",
        "τέλειοι",
        "τα",
        "ταξινομεί",
        "τεκμηριώνει",
        "τελειώσει",
        "τεσσαρών",
        "τοπικές",
        "τρέξει",
        "τρόπο",
        "τρόποι",
        "τύπου",
        "τύπους",
        "υπηρεσία",
        "υποψήφιο",
        "υψηλότερη",
        "υόρκη",
        "φίλος",
        "φαινόμενο",
        "φακέλους",
        "φράση",
        "χάος",
        "χαμηλός",
        "χαρακτηριστικό",
        "χαρακτηριστικών",
        "χαρτιού",
        "χειρότερα",
        "χρειάζονται",
        "χρησιμοποίησέ",
        "χρησιμοποιούνταν",
        "χρησιμοποιούσες",
        "χρησιμοποιώντας",
        "χρονοδιαγράμματα",
        "χρονοδιαγράμματος",
        "χρόνου",
        "χώρου",
        "ωραίο",
        "ύψος",
        "ώρα",
    )

    parts_of_speech: Dict[str, tuple] = {}



import re

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``vi_VN`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Vietnam
    """

    license_formats = ("##?-#####",)
    ascii_uppercase_vietnamese = "ABCDĐEFGHKLMNPSTUVXYZ"

    def license_plate(self) -> str:
        """Generate a license plate."""
        temp = re.sub(
            r"\?",
            lambda x: self.random_element(self.ascii_uppercase_vietnamese),
            self.random_element(self.license_formats),
        )
        return self.numerify(temp)



import re

from ipaddress import ip_address, ip_network
from itertools import cycle
from typing import Pattern
from unittest.mock import PropertyMock, patch

import pytest

from validators import domain as validate_domain
from validators import email as validate_email

from faker.providers.internet import Provider as InternetProvider
from faker.providers.internet.az_AZ import Provider as AzAzInternetProvider
from faker.providers.internet.en_GB import Provider as EnGbInternetProvider
from faker.providers.internet.es_ES import Provider as EsEsInternetProvider
from faker.providers.internet.pl_PL import Provider as PlPlInternetProvider
from faker.providers.internet.ro_RO import Provider as RoRoInternetProvider
from faker.providers.internet.ru_RU import Provider as RuRuInternetProvider
from faker.providers.internet.th_TH import Provider as ThThInternetProvider
from faker.providers.internet.zh_CN import Provider as ZhCnInternetProvider
from faker.providers.person.ja_JP import Provider as JaPersonProvider
from faker.utils import text


class TestInternetProvider:
    """Test internet provider methods"""

    num_samples = 100
    ipv4_pattern: Pattern = re.compile(
        r"^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}" r"(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$",
    )
    ipv4_network_pattern: Pattern = re.compile(
        r"^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}"
        r"(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)"
        r"/(?:\d|[12]\d|3[0-2])$",
    )

    def test_email(self, faker, num_samples):
        for _ in range(num_samples):
            email = faker.email()
            assert "@" in email

    def test_safe_default_email(self, faker, num_samples):
        expected_domains = ["example.com", "example.org", "example.net"]
        for _ in range(num_samples):
            email = faker.email()
            assert email.split("@")[1] in expected_domains

    def test_unsafe_email(self, faker, num_samples):
        not_expected_domains = ["example.com", "example.org", "example.net"]
        for _ in range(num_samples):
            email = faker.email(safe=False)
            assert email.split("@")[1] not in not_expected_domains

    def test_email_with_domain(self, faker):
        domain = "example.com"
        email = faker.email(domain=domain)
        assert email.split("@")[1] == domain

    def test_safe_email(self, faker, num_samples):
        expected_domains = ["example.com", "example.org", "example.net"]
        for _ in range(num_samples):
            email = faker.safe_email()
            assert email.split("@")[1] in expected_domains

    def test_safe_domain_names(self, faker, num_samples):
        expected_domains = ["example.com", "example.org", "example.net"]
        for _ in range(num_samples):
            safe_domain_name = faker.safe_domain_name()
            assert safe_domain_name in expected_domains

    @patch(
        "faker.providers.internet.Provider.image_placeholder_services",
        {"https://dummyimage.com/{width}x{height}"},
    )
    def test_image_url(self, faker):
        my_width = 500
        my_height = 1024
        url = faker.image_url(my_width, my_height)
        assert f"https://dummyimage.com/{my_width}x{my_height}" == url
        url = faker.image_url()
        assert "https://dummyimage.com/" in url
        url = faker.image_url(placeholder_url="https://example.com/{width}/height")
        assert url.startswith("https://example.com/")

    def test_hostname(self, faker):
        hostname_1_level = faker.hostname(levels=1)
        hostname_parts = hostname_1_level.split(".")
        assert hostname_1_level and isinstance(hostname_1_level, str)
        assert len(hostname_parts) == 3

        hostname_0_level = faker.hostname(levels=0)
        assert hostname_0_level and isinstance(hostname_0_level, str)

    def test_ipv4(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.ipv4()
            assert 7 <= len(address) <= 15
            assert self.ipv4_pattern.fullmatch(address)

        for _ in range(num_samples):
            address = faker.ipv4(network=True)
            assert 9 <= len(address) <= 18
            assert self.ipv4_network_pattern.fullmatch(address)

        for _ in range(num_samples):
            address = faker.ipv4(private=True)
            assert 7 <= len(address) <= 15
            assert self.ipv4_pattern.fullmatch(address)
            assert ip_address(address).is_private

        for _ in range(num_samples):
            address = faker.ipv4(private=False)
            assert 7 <= len(address) <= 15
            assert self.ipv4_pattern.fullmatch(address)
            assert not ip_address(address).is_private

    def test_ipv4_caching(self, faker):
        from faker.providers.internet import _IPv4Constants

        # The extra [None] here is to test code path involving whole IPv4 pool
        for address_class in list(_IPv4Constants._network_classes.keys()) + [None]:
            if address_class is None:
                networks_attr = "_cached_all_networks"
            else:
                networks_attr = f"_cached_all_class_{address_class}_networks"
            weights_attr = f"{networks_attr}_weights"
            provider = InternetProvider(faker)

            # First, test cache creation
            assert not hasattr(provider, networks_attr)
            assert not hasattr(provider, weights_attr)
            provider.ipv4(address_class=address_class)
            assert hasattr(provider, networks_attr)
            assert hasattr(provider, weights_attr)

            # Then, test cache access on subsequent calls
            with patch.object(
                InternetProvider, networks_attr, create=True, new_callable=PropertyMock
            ) as mock_networks_cache:
                with patch.object(
                    InternetProvider,
                    weights_attr,
                    create=True,
                    new_callable=PropertyMock,
                ) as mock_weights_cache:
                    # Keep test fast by patching the cache attributes to return something simple
                    mock_networks_cache.return_value = [ip_network("10.0.0.0/24")]
                    mock_weights_cache.return_value = [10]
                    for _ in range(100):
                        provider.ipv4(address_class=address_class)

                    # Python's hasattr() internally calls getattr()
                    # So each call to ipv4() accesses the cache attributes twice
                    assert mock_networks_cache.call_count == 200
                    assert mock_weights_cache.call_count == 200

    def test_ipv4_network_class(self, faker, num_samples):
        for _ in range(num_samples):
            klass = faker.ipv4_network_class()
            assert klass in "abc"

    def test_ipv4_private(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.ipv4_private()
            assert 7 <= len(address) <= 15
            assert self.ipv4_pattern.fullmatch(address)
            assert ip_address(address).is_private

        for _ in range(num_samples):
            address = faker.ipv4_private(network=True)
            assert 9 <= len(address) <= 18
            assert self.ipv4_network_pattern.fullmatch(address)
            assert ip_network(address)[0].is_private

    def test_ipv4_private_class(self, faker, num_samples):
        from faker.providers.internet import _IPv4Constants

        for clas in "abc":
            class_network = _IPv4Constants._network_classes[clas]
            class_min = class_network.network_address
            class_max = class_network.broadcast_address

            for _ in range(num_samples):
                address = faker.ipv4_private(address_class=clas)
                assert 7 <= len(address) <= 15
                assert self.ipv4_pattern.fullmatch(address)
                assert ip_address(address).is_private
                assert class_min <= ip_address(address) <= class_max

    def test_ipv4_public_caching(self, faker):
        from faker.providers.internet import _IPv4Constants

        for address_class in _IPv4Constants._network_classes.keys():
            networks_attr = f"_cached_public_class_{address_class}_networks"
            weights_attr = f"{networks_attr}_weights"
            provider = InternetProvider(faker)

            # First, test cache creation
            assert not hasattr(provider, networks_attr)
            assert not hasattr(provider, weights_attr)
            provider.ipv4_public(address_class=address_class)
            assert hasattr(provider, networks_attr)
            assert hasattr(provider, weights_attr)

            # Then, test cache access on subsequent calls
            with patch.object(
                InternetProvider, networks_attr, create=True, new_callable=PropertyMock
            ) as mock_networks_cache:
                with patch.object(
                    InternetProvider,
                    weights_attr,
                    create=True,
                    new_callable=PropertyMock,
                ) as mock_weights_cache:
                    # Keep test fast by patching the cache attributes to return something simple
                    mock_networks_cache.return_value = [ip_network("10.0.0.0/24")]
                    mock_weights_cache.return_value = [10]
                    for _ in range(100):
                        provider.ipv4_public(address_class=address_class)

                    # Python's hasattr() internally calls getattr()
                    # So each call to ipv4_public() accesses the cache attributes twice
                    assert mock_networks_cache.call_count == 200
                    assert mock_weights_cache.call_count == 200

    def test_ipv4_public(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.ipv4_public()
            assert 7 <= len(address) <= 15
            assert self.ipv4_pattern.fullmatch(address)
            assert not ip_address(address).is_private

        for _ in range(num_samples):
            address = faker.ipv4_public(network=True)
            assert 9 <= len(address) <= 18
            assert self.ipv4_network_pattern.fullmatch(address)
            # Hack around ipaddress module
            # As 192.0.0.0 is net addr of many 192.0.0.0/* nets
            # ipaddress considers them as private
            if ip_network(address).network_address != ip_address("192.0.0.0"):
                assert not ip_network(address)[0].is_private

    def test_ipv4_public_class(self, faker, num_samples):
        from faker.providers.internet import _IPv4Constants

        for clas in "abc":
            class_network = _IPv4Constants._network_classes[clas]
            class_min = class_network.network_address
            class_max = class_network.broadcast_address

            for _ in range(num_samples):
                address = faker.ipv4_public(address_class=clas)
                assert 7 <= len(address) <= 15
                assert not ip_address(address).is_private
                assert class_min <= ip_address(address) <= class_max
                assert self.ipv4_pattern.fullmatch(address)

    def test_ipv4_distribution_selection(self):
        from faker.generator import Generator, random
        from faker.utils.distribution import choices_distribution

        provider = InternetProvider(Generator())

        subnets = [ip_network("10.0.0.0/8"), ip_network("11.0.0.0/8")]
        valid_weights = [1, 1]
        list_of_invalid_weights = [
            [1, 2, 3],  # List size does not match subnet list size
            ["a", "b"],  # List size matches, but elements are invalid
            11,  # Not a list or valid iterable
        ]

        with patch("faker.providers.internet.choices_distribution", wraps=choices_distribution) as mock_choices_fn:
            with patch("faker.generator.random.choice", wraps=random.choice) as mock_random_choice:
                # If weights argument is valid, only `choices_distribution` should be called
                provider._random_ipv4_address_from_subnets(subnets, valid_weights)
                assert mock_choices_fn.call_count == 1
                assert mock_random_choice.call_count == 0

                # If weights argument is invalid, calls to `choices_distribution` will fail
                # and calls to `random.choice` will be made as failover behavior
                for invalid_weights in list_of_invalid_weights:
                    # Reset mock objects for each iteration
                    mock_random_choice.reset_mock()
                    mock_choices_fn.reset_mock()

                    provider._random_ipv4_address_from_subnets(subnets, invalid_weights)
                    assert mock_choices_fn.call_count == 0
                    assert mock_random_choice.call_count == 1

    def test_ipv6(self, faker, num_samples):
        provider = InternetProvider(faker)

        for _ in range(num_samples):
            address = provider.ipv6()
            assert len(address) >= 3  # ::1
            assert len(address) <= 39
            assert re.compile(r"^([0-9a-f]{0,4}:){2,7}[0-9a-f]{1,4}$").search(address)

        for _ in range(num_samples):
            address = provider.ipv6(network=True)
            assert len(address) >= 4  # ::/8
            assert len(address) <= 39 + 4
            assert re.compile(r"^([0-9a-f]{0,4}:){2,7}[0-9a-f]{0,4}/\d{1,3}$").search(address)

    def test_mac_address(self, faker):
        provider = InternetProvider(faker)

        unicast_address = provider.mac_address()
        assert int(unicast_address[0:2], base=16) % 2 == 0

        multicast_address = provider.mac_address(multicast=True)
        assert int(multicast_address[0:2], base=16) % 2 == 1

    def test_port_number(self, faker, num_samples):
        for _ in range(num_samples):
            assert 0 <= faker.port_number() <= 65535
            assert 0 <= faker.port_number(is_system=True) <= 1023
            assert 1024 <= faker.port_number(is_user=True) <= 49151
            assert 49152 <= faker.port_number(is_dynamic=True) <= 65535

    def test_http_method(self, faker, num_samples):
        expected_methods = [
            "CONNECT",
            "DELETE",
            "GET",
            "HEAD",
            "OPTIONS",
            "PATCH",
            "POST",
            "PUT",
            "TRACE",
        ]

        got_methods = set()
        for _ in range(num_samples):
            got_methods.add(faker.http_method())

        assert expected_methods == sorted(got_methods)

    def test_http_status_code(self, faker, num_samples):
        provider = InternetProvider(faker)
        status_code = provider.http_status_code()
        assert isinstance(status_code, int)
        assert 100 <= status_code <= 599
        status_code = provider.http_status_code(include_unassigned=False)
        assert isinstance(status_code, int)
        assert 100 <= status_code <= 599
        assert status_code in InternetProvider.http_assigned_codes

    def test_dga(self, faker):
        assert faker.dga() != faker.dga()

        expected_domain = "cqphixmpdfpptskr.com"
        assert faker.dga(day=1, month=1, year=1000, tld="com", length=16) == expected_domain

    def test_iana_id(self, faker, num_samples):
        for _ in range(num_samples):
            assert 1 <= int(faker.iana_id()) <= 8888888

    def test_ripe_id(self, faker, num_samples):
        pattern: Pattern = re.compile(r"^ORG-[A-Z]{2,4}[1-9]\d{0,4}-RIPE$")
        for _ in range(num_samples):
            assert pattern.fullmatch(faker.ripe_id())

    def test_nic_handles(self, faker, num_samples):
        pattern: Pattern = re.compile(r"^[A-Z]{2,4}[1-9]\d{0,4}-[A-Z]*")
        for _ in range(num_samples):
            nhs = faker.nic_handles()
            for nh in nhs:
                assert pattern.fullmatch(nh)

        nhs = faker.nic_handles(suffix="??", count=num_samples)
        assert len(nhs) == num_samples
        for nh in nhs:
            assert pattern.fullmatch(nh)

        with pytest.raises(ValueError):
            faker.nic_handles(suffix="")


class TestInternetProviderUrl:
    """Test internet url generation"""

    @staticmethod
    def is_correct_scheme(url, schemes):
        return any(url.startswith(f"{scheme}://") for scheme in schemes)

    def test_url_default_schemes(self, faker):
        for _ in range(100):
            url = faker.url()
            assert self.is_correct_scheme(url, ["http", "https"])

    def test_url_custom_schemes(self, faker):
        schemes_sets = [
            ["usb"],
            ["ftp", "file"],
            ["usb", "telnet", "http"],
        ]
        for _, schemes in zip(range(100), cycle(schemes_sets)):
            url = faker.url(schemes=schemes)
            assert self.is_correct_scheme(url, schemes)

    def test_url_empty_schemes_list_generate_schemeless_urls(self, faker):
        for _ in range(100):
            url = faker.url(schemes=[])
            assert not url.startswith("http")
            assert url.startswith("://")


class TestInternetProviderUri:
    """Test internet uri generation"""

    @staticmethod
    def is_correct_scheme(uri, schemes):
        return any(uri.startswith(f"{scheme}://") for scheme in schemes)

    def test_uri_default_schemes(self, faker):
        for _ in range(100):
            uri = faker.uri()
            assert self.is_correct_scheme(uri, ["http", "https"])

    def test_uri_custom_schemes(self, faker):
        schemes_sets = [
            ["usb"],
            ["ftp", "file"],
            ["usb", "telnet", "http"],
        ]
        for _, schemes in zip(range(100), cycle(schemes_sets)):
            uri = faker.uri(schemes=schemes)
            assert self.is_correct_scheme(uri, schemes)

    def test_uri_empty_schemes_list_generate_schemeless_urls(self, faker):
        for _ in range(100):
            uri = faker.uri(schemes=[])
            assert not uri.startswith("http")
            assert uri.startswith("://")

    def test_uri_extension(self, faker):
        uri = faker.uri()
        assert "." in uri

    def test_uri_component(self, faker):
        uri = faker.uri()
        assert "/" in uri

    def test_uri_deep(self, faker):
        uri = faker.uri(deep=1).replace("://", "")
        assert uri.count("/") == 1

        uri = faker.uri(deep=3).replace("://", "")
        assert uri.count("/") == 3


class TestJaJp:
    """Test ja_JP internet provider methods"""

    def test_internet(self, faker):
        names = JaPersonProvider.last_romanized_names

        domain_word = faker.domain_word()
        assert isinstance(domain_word, str)
        assert any(domain_word == text.slugify(name) for name in names)

        domain_name = faker.domain_name()
        deep_domain_name = faker.domain_name(3)
        assert isinstance(domain_name, str)
        assert isinstance(deep_domain_name, str)
        assert deep_domain_name.count(".") == 3
        with pytest.raises(ValueError):
            faker.domain_name(-1)

        user_name = faker.user_name()
        assert isinstance(user_name, str)

        tld = faker.tld()
        assert isinstance(tld, str)


class TestZhCn:
    """Test zh_CN internet provider methods"""

    def test_email(self, faker):
        email = faker.email()
        validate_email(email)

    def test_domain_word(self, faker):
        domain_word = faker.domain_word()
        assert len(domain_word) > 1

    @patch(
        "faker.providers.internet.Provider.tld",
        lambda x: "cn",
    )
    def test_domain_name(self, faker):
        domain_name_1_level = faker.domain_name(levels=1)
        domain_parts = domain_name_1_level.split(".")
        assert len(domain_parts) == 2
        assert domain_parts[-1] == "cn"
        domain_name_2_level = faker.domain_name(levels=2)
        domain_parts = domain_name_2_level.split(".")
        assert len(domain_parts) == 3
        assert domain_parts[-1] == "cn"
        assert domain_parts[1] in [
            "ac",
            "com",
            "edu",
            "gov",
            "mil",
            "net",
            "org",
            "ah",
            "bj",
            "cq",
            "fj",
            "gd",
            "gs",
            "gz",
            "gx",
            "ha",
            "hb",
            "he",
            "hi",
            "hk",
            "hl",
            "hn",
            "jl",
            "js",
            "jx",
            "ln",
            "mo",
            "nm",
            "nx",
            "qh",
            "sc",
            "sd",
            "sh",
            "sn",
            "sx",
            "tj",
            "xj",
            "xz",
            "yn",
            "zj",
        ]

    def test_domain_name_one_level_after_tld(self, faker):
        provider = ZhCnInternetProvider(faker)
        for _ in range(100):
            domain_name = faker.domain_name(levels=1)
            domain_parts = domain_name.split(".")
            assert len(domain_parts) == 2
            assert domain_parts[-1] in provider.tlds.keys()
            assert domain_parts[0] not in provider.second_level_domains

    @patch("faker.providers.internet.zh_CN.Provider.domain_word")
    @patch("faker.providers.internet.Provider.tld")
    def test_domain_name_two_levels_after_cn_tld(self, mock_tld, mock_domain_word, faker):
        provider = ZhCnInternetProvider(faker)

        # If tld() returns cn, second level name should be selected from second_level_domains
        # and domain_word() will only be called once which will be used for the third level
        mock_tld.return_value = "cn"
        mock_domain_word.return_value = "li"
        for _ in range(100):
            mock_domain_word.reset_mock()
            domain_name = faker.domain_name(levels=2)
            domain_parts = domain_name.split(".")
            assert len(domain_parts) == 3
            assert domain_parts[-1] == "cn"
            assert domain_parts[-2] in provider.second_level_domains
            assert domain_parts[0] == "li"
            assert mock_domain_word.call_count == 1

    @patch("faker.providers.internet.zh_CN.Provider.domain_word")
    @patch("faker.providers.internet.Provider.tld")
    def test_domain_name_two_levels_after_non_cn_tld(self, mock_tld, mock_domain_word, faker):
        # If tld() does not return cn, domain_word() will be called twice
        mock_domain_word.reset_mock()
        mock_tld.return_value = "net"
        mock_domain_word.return_value = "li"
        domain_name = faker.domain_name(levels=2)
        assert domain_name == "li.li.net"
        assert mock_domain_word.call_count == 2

    @patch("faker.providers.internet.zh_CN.Provider.domain_word")
    @patch("faker.providers.internet.Provider.tld")
    def test_domain_name_more_than_two_levels_after_cn_tld(self, mock_tld, mock_domain_word, faker):
        provider = ZhCnInternetProvider(faker)

        mock_tld.return_value = "cn"
        mock_domain_word.return_value = "li"
        for levels in range(3, 10):
            with patch(
                "faker.providers.internet.zh_CN.Provider.domain_name",
                wraps=faker.domain_name,
            ) as mock_domain_name:
                mock_tld.reset_mock()
                mock_domain_word.reset_mock()
                mock_domain_name.reset_mock()
                domain_name = faker.domain_name(levels=levels)
                domain_parts = domain_name.split(".")

                # Same assertions as levels=2 for tld and second level if tld is cn
                # But every level henceforth should return the mocked value
                assert domain_parts[-1] == "cn"
                assert domain_parts[-2] in provider.second_level_domains
                assert all(domain_part == "li" for domain_part in domain_parts[:-2])

                # tld() method should only be called once, domain_word() will be called for each
                # level after tld except the second, and recursive calls to domain_name() will be
                # made for each level starting from the third level after tld
                assert mock_tld.call_count == 1
                assert mock_domain_word.call_count == levels - 1
                assert mock_domain_name.call_count == levels - 2

    @patch("faker.providers.internet.zh_CN.Provider.domain_word")
    @patch("faker.providers.internet.Provider.tld")
    def test_domain_name_more_than_two_levels_after_non_cn_tld(self, mock_tld, mock_domain_word, faker):
        mock_tld.return_value = "net"
        mock_domain_word.return_value = "li"
        for levels in range(3, 10):
            with patch(
                "faker.providers.internet.zh_CN.Provider.domain_name",
                wraps=faker.domain_name,
            ) as mock_domain_name:
                mock_tld.reset_mock()
                mock_domain_word.reset_mock()
                mock_domain_name.reset_mock()
                domain_name = faker.domain_name(levels=levels)
                domain_parts = domain_name.split(".")

                # Same assertions as levels=2 for non cn tld and
                # every level henceforth should return the mocked value
                assert domain_parts[-1] == "net"
                assert all(domain_part == "li" for domain_part in domain_parts[:-1])

                # tld() method should only be called once, domain_word() will be called for each
                # level after tld, and recursive calls to domain_name() will be made for each
                # level starting from the third level after tld
                assert mock_tld.call_count == 1
                assert mock_domain_word.call_count == levels
                assert mock_domain_name.call_count == levels - 2

    def test_domain_name_bad_level(self, faker):
        with pytest.raises(ValueError):
            faker.domain_name(levels=0)


class TestZhTw:
    """Test zh_TW internet provider methods"""

    def test_email(self, faker):
        email = faker.email()
        validate_email(email)


class TestHuHu:
    """Test hu_HU internet provider methods"""

    def test_internet(self, faker):
        domain_name = faker.domain_name()
        assert isinstance(domain_name, str)
        tld = faker.tld()
        assert isinstance(tld, str)
        email = faker.email()
        assert isinstance(email, str)


class TestPlPl:
    """Test pl_PL internet provider methods"""

    def test_free_email_domain(self, faker):
        domain = faker.free_email_domain()
        assert domain in PlPlInternetProvider.free_email_domains

    def test_tld(self, faker):
        tld = faker.tld()
        assert tld in PlPlInternetProvider.tlds


class TestNlNl:
    """Test nl_NL internet provider methods"""

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "fabiënné",
    )
    def test_ascii_safe_email(self, faker):
        email = faker.ascii_safe_email()
        validate_email(email)
        assert email.split("@")[0] == "fabienne"

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "fabiënné",
    )
    def test_ascii_free_email(self, faker):
        email = faker.ascii_free_email()
        validate_email(email)
        assert email.split("@")[0] == "fabienne"

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "fabiënné",
    )
    def test_ascii_company_email(self, faker):
        email = faker.ascii_company_email()
        validate_email(email)
        assert email.split("@")[0] == "fabienne"


class TestArAa:
    """Test ar_AA internet provider methods"""

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "اصيل",
    )
    def test_ascii_safe_email(self, faker):
        email = faker.ascii_safe_email()
        validate_email(email)
        assert email.split("@")[0] == "asyl"

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "اصيل",
    )
    def test_ascii_free_email(self, faker):
        email = faker.ascii_free_email()
        validate_email(email)
        assert email.split("@")[0] == "asyl"

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "اصيل",
    )
    def test_ascii_company_email(self, faker):
        email = faker.ascii_company_email()
        validate_email(email)
        assert email.split("@")[0] == "asyl"


class TestAzAz:
    """Test az_AZ internet provider methods"""

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "AğamüşviqƏlövsətov",
    )
    def test_ascii_free_email(self, faker):
        email = faker.ascii_free_email()
        validate_email(email)
        assert email.split("@")[0] == "agamushviqelovsetov"

    def test_free_email_domain(self, faker):
        domain = faker.free_email_domain()
        assert domain in AzAzInternetProvider.free_email_domains

    def test_tld(self, faker):
        tld = faker.tld()
        assert tld in AzAzInternetProvider.tlds


class TestPtBr:
    """Test pt_BR internet provider methods"""

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "VitóriaMagalhães",
    )
    def test_ascii_safe_email(self, faker):
        email = faker.ascii_safe_email()
        validate_email(email)
        assert email.split("@")[0] == "vitoriamagalhaes"

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "JoãoSimões",
    )
    def test_ascii_free_email(self, faker):
        email = faker.ascii_free_email()
        validate_email(email)
        assert email.split("@")[0] == "joaosimoes"

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "AndréCauã",
    )
    def test_ascii_company_email(self, faker):
        email = faker.ascii_company_email()
        validate_email(email)
        assert email.split("@")[0] == "andrecaua"


class TestEnPh:
    """Test en_PH internet provider methods"""

    num_samples = 100

    def test_domain_name(self, faker, num_samples):
        for i in range(num_samples):
            domain = faker.domain_name()
            validate_domain(domain)


class TestFilPh(TestEnPh):
    """Test fil_PH internet provider methods"""

    pass


class TestTlPh(TestFilPh):
    """Test tl_PH internet provider methods"""

    pass


class TestEnGb:
    """Tests for the en_GB locale."""

    def test_free_email_domain(self, faker):
        domain = faker.free_email_domain()
        assert domain in EnGbInternetProvider.free_email_domains

    def test_tld(self, faker):
        tld = faker.tld()
        assert tld in EnGbInternetProvider.tlds


class TestEsEs:
    """Tests for the es_ES locale."""

    def test_tld(self, faker):
        tld = faker.tld()
        assert tld in EsEsInternetProvider.tlds


class TestRoRo:
    """Test ro_RO internet provider methods"""

    def test_free_email_domain(self, faker):
        domain = faker.free_email_domain()
        assert domain in RoRoInternetProvider.free_email_domains

    def test_tld(self, faker):
        tld = faker.tld()
        assert tld in PlPlInternetProvider.tlds


class TestRuRu:
    """Test ru_RU internet provider methods"""

    def test_free_email_domain(self, faker):
        assert faker.free_email_domain() in RuRuInternetProvider.free_email_domains

    def test_tld(self, faker):
        assert faker.tld() in RuRuInternetProvider.tlds

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "ИванИванов",
    )
    def test_ascii_safe_email(self, faker):
        email = faker.ascii_safe_email()
        validate_email(email)
        assert email.split("@")[0] == "ivanivanov"

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "АлександрСмирнов",
    )
    def test_ascii_free_email(self, faker):
        email = faker.ascii_free_email()
        validate_email(email)
        assert email.split("@")[0] == "aleksandrsmirnov"

    @patch(
        "faker.providers.internet.Provider.user_name",
        lambda x: "СергейКузнецов",
    )
    def test_ascii_company_email(self, faker):
        email = faker.ascii_company_email()
        validate_email(email)
        assert email.split("@")[0] == "sergekuznetsov"


class TestThTh:
    """Test th_TH internet provider methods"""

    def test_tld(self, faker):
        tld = faker.tld()
        assert tld in ThThInternetProvider.tlds



import io
import string
import sys
import unittest

from unittest.mock import MagicMock, patch

import pytest

from faker import Faker, Generator
from faker.factory import Factory
from faker.generator import random
from faker.utils import decorators, text


class FactoryTestCase(unittest.TestCase):
    def setUp(self):
        self.generator = Generator()

    def test_documentor(self):
        from faker.cli import print_doc

        output = io.StringIO()
        print_doc(output=output)
        print_doc("address", output=output)
        print_doc("faker.providers.person.it_IT", output=output)
        assert output.getvalue()

    def test_command(self):
        from faker.cli import Command

        orig_stdout = sys.stdout
        try:
            sys.stdout = io.StringIO()
            command = Command(["faker", "address"])
            command.execute()
            assert sys.stdout.getvalue()
        finally:
            sys.stdout = orig_stdout

    def test_command_custom_provider(self):
        from faker.cli import Command

        orig_stdout = sys.stdout
        try:
            sys.stdout = io.StringIO()
            command = Command(["faker", "foo", "-i", "tests.mymodule.en_US"])
            command.execute()
            assert sys.stdout.getvalue()
        finally:
            sys.stdout = orig_stdout

    def test_cli_seed(self):
        from faker.cli import Command

        orig_stdout = sys.stdout
        try:
            sys.stdout = io.StringIO()
            base_args = ["faker", "address"]
            target_args = ["--seed", "967"]
            commands = [
                Command(base_args + target_args),
                Command(base_args + target_args),
            ]
            cli_output = [None] * 2
            for i in range(2):
                commands[i].execute()
                cli_output[i] = sys.stdout.getvalue()
            cli_output[1] = cli_output[1][len(cli_output[0]) :]
            assert cli_output[0][:10] == cli_output[1][:10]
        finally:
            sys.stdout = orig_stdout

    def test_cli_seed_with_repeat(self):
        from faker.cli import Command

        orig_stdout = sys.stdout
        try:
            sys.stdout = io.StringIO()
            base_args = ["faker", "address", "-r", "3"]
            target_args = ["--seed", "967"]
            commands = [
                Command(base_args + target_args),
                Command(base_args + target_args),
            ]
            cli_output = [None] * 2
            for i in range(2):
                commands[i].execute()
                cli_output[i] = sys.stdout.getvalue()
            cli_output[1] = cli_output[1][len(cli_output[0]) :]
            assert cli_output[0] == cli_output[1]
        finally:
            sys.stdout = orig_stdout

    def test_cli_verbosity(self):
        from faker.cli import Command

        orig_stdout = sys.stdout
        try:
            sys.stdout = io.StringIO()
            base_args = ["faker", "address", "--seed", "769"]
            target_args = ["-v"]
            commands = [Command(base_args), Command(base_args + target_args)]
            cli_output = [None] * 2
            for i in range(2):
                commands[i].execute()
                cli_output[i] = sys.stdout.getvalue()
            simple_output, verbose_output = cli_output
            assert simple_output != verbose_output
        finally:
            sys.stdout = orig_stdout

    def test_unknown_provider(self):
        with pytest.raises(ModuleNotFoundError) as excinfo:
            Factory.create(providers=["dummy_provider"])
        assert str(excinfo.value) == "No module named 'dummy_provider'"

        with pytest.raises(ModuleNotFoundError) as excinfo:
            Factory.create(providers=["dummy_provider"], locale="it_IT")
        assert str(excinfo.value) == "No module named 'dummy_provider'"

    def test_unknown_locale(self):
        with pytest.raises(AttributeError) as excinfo:
            Factory.create(locale="77")
        assert str(excinfo.value) == "Invalid configuration for faker locale `77`"

        with pytest.raises(AttributeError) as excinfo:
            Factory.create(locale="77_US")
        assert str(excinfo.value) == "Invalid configuration for faker locale `77_US`"

    def test_lang_unlocalized_provider(self):
        for locale in (None, "", "en_GB", "it_IT"):
            factory = Factory.create(providers=["faker.providers.file"], locale=locale)
            assert len(factory.providers) == 1
            assert factory.providers[0].__provider__ == "faker.providers.file"
            assert factory.providers[0].__lang__ is None

    def test_lang_localized_provider(self, with_default=True):
        class DummyProviderModule:
            localized = True

            def __init__(self):
                if with_default:
                    self.default_locale = "ar_EG"

            @property
            def __name__(self):
                return self.__class__.__name__

            class Provider:
                def __init__(self, *args, **kwargs):
                    pass

        with patch.multiple(
            "faker.factory",
            import_module=MagicMock(return_value=DummyProviderModule()),
            list_module=MagicMock(return_value=("en_GB", "it_IT")),
            DEFAULT_LOCALE="ko_KR",
        ):
            test_cases = [
                (None, False),
                ("", False),
                ("ar", False),
                ("es_CO", False),
                ("en", False),
                ("en_GB", True),
                ("ar_EG", with_default),  # True if module defines a default locale
            ]
            for locale, expected_used in test_cases:
                factory = Factory.create(providers=["dummy"], locale=locale)
                assert factory.providers[0].__provider__ == "dummy"
                from faker.config import DEFAULT_LOCALE

                print(f"requested locale = {locale} , DEFAULT LOCALE {DEFAULT_LOCALE}")
                expected_locale = locale if expected_used else ("ar_EG" if with_default else "ko_KR")
                assert factory.providers[0].__lang__ == expected_locale

    def test_lang_localized_provider_without_default(self):
        self.test_lang_localized_provider(with_default=False)

    def test_slugify(self):
        slug = text.slugify("a'b/c")
        assert slug == "abc"

        slug = text.slugify("àeìöú")
        assert slug == "aeiou"

        slug = text.slugify("àeì.öú")
        assert slug == "aeiou"

        slug = text.slugify("àeì.öú", allow_dots=True)
        assert slug == "aei.ou"

        slug = text.slugify("àeì.öú", allow_unicode=True)
        assert slug == "àeìöú"

        slug = text.slugify("àeì.öú", allow_unicode=True, allow_dots=True)
        assert slug == "àeì.öú"

        @decorators.slugify
        def fn(s):
            return s

        slug = fn("a'b/c")
        assert slug == "abc"

        @decorators.slugify_domain
        def fn(s):
            return s

        slug = fn("a'b/.c")
        assert slug == "ab.c"

        @decorators.slugify_unicode
        def fn(s):
            return s

        slug = fn("a'b/.cé")
        assert slug == "abcé"

    def test_binary(self):
        from faker.providers.misc import Provider

        provider = Provider(self.generator)

        for _ in range(999):
            length = random.randint(0, 2**10)
            binary = provider.binary(length)

            assert isinstance(binary, (bytes, bytearray))
            assert len(binary) == length

        for _ in range(999):
            self.generator.seed(_)
            binary1 = provider.binary(_)
            self.generator.seed(_)
            binary2 = provider.binary(_)

            assert binary1 == binary2

    def test_password(self):
        from faker.providers.misc import Provider

        provider = Provider(self.generator)

        def in_string(char, _str):
            return char in _str

        for _ in range(999):
            password = provider.password()

            assert any(in_string(char, password) for char in "!@#$%^&*()_+")
            assert any(in_string(char, password) for char in string.digits)
            assert any(in_string(char, password) for char in string.ascii_uppercase)
            assert any(in_string(char, password) for char in string.ascii_lowercase)

        with pytest.raises(AssertionError):
            provider.password(length=2)

    def test_prefix_suffix_always_string(self):
        # Locales known to contain `*_male` and `*_female`.
        for locale in ("bg_BG", "dk_DK", "en", "ru_RU", "tr_TR"):
            fake = Faker(locale=locale)
            for x in range(20):  # Probabilistic testing.
                self.assertIsInstance(fake.prefix(), str)
                self.assertIsInstance(fake.suffix(), str)

    def test_random_pystr_characters(self):
        from faker.providers.python import Provider

        provider = Provider(self.generator)

        characters = provider.pystr()
        assert len(characters) == 20
        characters = provider.pystr(max_chars=255)
        assert len(characters) == 255
        characters = provider.pystr(max_chars=0)
        assert characters == ""
        characters = provider.pystr(max_chars=-10)
        assert characters == ""
        characters = provider.pystr(min_chars=10, max_chars=255)
        assert len(characters) >= 10

    def test_random_pyfloat(self):
        from faker.providers.python import Provider

        provider = Provider(self.generator)

        assert 0 <= abs(provider.pyfloat(left_digits=1)) < 10
        assert 0 <= abs(provider.pyfloat(left_digits=0)) < 1
        x = abs(provider.pyfloat(right_digits=0))
        assert x - int(x) == 0
        with pytest.raises(ValueError):
            provider.pyfloat(left_digits=0, right_digits=0)

    def test_pyfloat_in_range(self):
        # tests for https://github.com/joke2k/faker/issues/994
        fake = Faker()

        for i in range(20):
            for min_value, max_value in [
                (0, 1),
                (-1, 1),
                (None, -5),
                (-5, None),
                (None, 5),
                (5, None),
            ]:
                fake.seed_instance(i)
                result = fake.pyfloat(min_value=min_value, max_value=max_value)
                if min_value is not None:
                    assert result >= min_value
                if max_value is not None:
                    assert result <= max_value

    def test_negative_pyfloat(self):
        # tests for https://github.com/joke2k/faker/issues/813
        fake = Faker()
        fake.seed_instance(32167)
        assert any(fake.pyfloat(left_digits=0, positive=False) < 0 for _ in range(100))
        assert any(fake.pydecimal(left_digits=0, positive=False) < 0 for _ in range(100))

    def test_arbitrary_digits_pydecimal(self):
        # tests for https://github.com/joke2k/faker/issues/1462
        fake = Faker()
        assert any(
            len(str(fake.pydecimal(left_digits=sys.float_info.dig + i))) > sys.float_info.dig for i in range(100)
        )
        assert any(len(str(fake.pydecimal())) > sys.float_info.dig for _ in range(100))

    def test_pyfloat_empty_range_error(self):
        # tests for https://github.com/joke2k/faker/issues/1048
        fake = Faker()
        fake.seed_instance(8038)
        assert fake.pyfloat(max_value=9999) < 9999

    def test_pyfloat_same_min_max(self):
        # tests for https://github.com/joke2k/faker/issues/1048
        fake = Faker()
        with pytest.raises(ValueError):
            assert fake.pyfloat(min_value=9999, max_value=9999)

    def test_instance_seed_chain(self):
        factory = Factory.create()

        names = ["Real Name0", "Real Name1", "Real Name2", "Real Name0", "Real Name2"]
        anonymized = [factory.seed_instance(name).name() for name in names]
        assert anonymized[0] == anonymized[3]
        assert anonymized[2] == anonymized[4]


if __name__ == "__main__":
    unittest.main()  # pragma: no cover



import json
import unittest

from importlib import import_module
from pathlib import Path

import faker

from faker.config import META_PROVIDERS_MODULES, PROVIDERS
from faker.generator import random
from faker.typing import OrderedDictType
from faker.utils.checksums import calculate_luhn, luhn_checksum
from faker.utils.datasets import add_ordereddicts
from faker.utils.distribution import choices_distribution, choices_distribution_unique
from faker.utils.loading import find_available_locales, find_available_providers, get_path

TEST_DIR = Path(__file__).resolve().parent


class UtilsTestCase(unittest.TestCase):
    def test_choice_distribution(self):
        a = ("a", "b", "c", "d")
        p = (0.5, 0.2, 0.2, 0.1)

        sample = choices_distribution(a, p)[0]
        assert sample in a

        random_state = json.loads((TEST_DIR / "random_state.json").read_text())
        random_state[1] = tuple(random_state[1])

        random.setstate(random_state)
        samples = choices_distribution(a, p, length=100)
        a_pop = len([i for i in samples if i == "a"])
        b_pop = len([i for i in samples if i == "b"])
        c_pop = len([i for i in samples if i == "c"])
        d_pop = len([i for i in samples if i == "d"])

        boundaries = []
        tolerance = 5
        for probability in p:
            boundaries.append([100 * probability + tolerance, 100 * probability - tolerance])

        assert boundaries[0][0] > a_pop > boundaries[0][1]
        assert boundaries[1][0] > b_pop > boundaries[1][1]
        assert boundaries[2][0] > c_pop > boundaries[2][1]
        assert boundaries[3][0] > d_pop > boundaries[3][1]

    def test_choices_distribution_unique(self):
        a = ("a", "b", "c", "d")
        p = (0.25, 0.25, 0.25, 0.25)
        with self.assertRaises(AssertionError):
            choices_distribution_unique(a, p, length=5)

        samples = choices_distribution_unique(a, p, length=4)
        assert len(set(samples)) == len(samples)

    def test_get_path(self):
        result = get_path(faker)
        assert isinstance(result, str)

    def test_find_available_locales(self):
        result = find_available_locales(PROVIDERS)
        assert len(result) != 0

    def test_find_available_providers(self):
        modules = [import_module(path) for path in META_PROVIDERS_MODULES]
        providers = find_available_providers(modules)

        expected_providers = list(
            map(
                str,
                [
                    "faker.providers.address",
                    "faker.providers.automotive",
                    "faker.providers.bank",
                    "faker.providers.barcode",
                    "faker.providers.color",
                    "faker.providers.company",
                    "faker.providers.credit_card",
                    "faker.providers.currency",
                    "faker.providers.date_time",
                    "faker.providers.emoji",
                    "faker.providers.file",
                    "faker.providers.geo",
                    "faker.providers.internet",
                    "faker.providers.isbn",
                    "faker.providers.job",
                    "faker.providers.lorem",
                    "faker.providers.misc",
                    "faker.providers.passport",
                    "faker.providers.person",
                    "faker.providers.phone_number",
                    "faker.providers.profile",
                    "faker.providers.python",
                    "faker.providers.sbn",
                    "faker.providers.ssn",
                    "faker.providers.user_agent",
                ],
            )
        )
        assert providers == expected_providers

    def test_luhn_checksum(self):
        """
        Tests if a valid checksum is generated
        Example from wiki: https://en.wikipedia.org/wiki/Luhn_algorithm
        """
        check_digit = calculate_luhn("7992739871")
        assert check_digit == 3

    def test_valid_luhn(self):
        """
        Tests if the number has a valid check digit
        Example from wiki https://en.wikipedia.org/wiki/Luhn_algorithm
        """
        assert luhn_checksum("79927398713") == 0

    def test_invalid_luhn(self):
        """
        Tests a number with an invalid check digit
        Example from wiki https://en.wikipedia.org/wiki/Luhn_algorithm
        """
        assert luhn_checksum("79927398714") != 0

    def test_add_ordereddicts(self):
        d1 = OrderedDictType([("a", 1), ("b", 2)])
        d2 = OrderedDictType([("c", 3), ("d", 4)])
        result = add_ordereddicts(d1, d2)
        assert result == OrderedDictType([("a", 1), ("b", 2), ("c", 3), ("d", 4)])



from __future__ import annotations

import copy
import functools
import re

from collections import OrderedDict
from random import Random
from typing import Any, Callable, Pattern, Sequence, TypeVar

from .config import DEFAULT_LOCALE
from .exceptions import UniquenessException
from .factory import Factory
from .generator import Generator, random
from .typing import SeedType
from .utils.distribution import choices_distribution

_UNIQUE_ATTEMPTS = 1000

RetType = TypeVar("RetType")


class Faker:
    """Proxy class capable of supporting multiple locales"""

    cache_pattern: Pattern = re.compile(r"^_cached_\w*_mapping$")
    generator_attrs = [
        attr for attr in dir(Generator) if not attr.startswith("__") and attr not in ["seed", "seed_instance", "random"]
    ]

    def __init__(
        self,
        locale: str | Sequence[str] | dict[str, int | float] | None = None,
        providers: list[str] | None = None,
        generator: Generator | None = None,
        includes: list[str] | None = None,
        use_weighting: bool = True,
        **config: Any,
    ) -> None:
        self._factory_map: OrderedDict[str, Generator | Faker] = OrderedDict()
        self._weights = None
        self._unique_proxy = UniqueProxy(self)
        self._optional_proxy = OptionalProxy(self)

        if isinstance(locale, str):
            locales = [locale.replace("-", "_")]

        # This guarantees a FIFO ordering of elements in `locales` based on the final
        # locale string while discarding duplicates after processing
        elif isinstance(locale, (list, tuple, set)):
            locales = []
            for code in locale:
                if not isinstance(code, str):
                    raise TypeError(f'The locale "{str(code)}" must be a string.')
                final_locale = code.replace("-", "_")
                if final_locale not in locales:
                    locales.append(final_locale)

        elif isinstance(locale, (OrderedDict, dict)):
            assert all(isinstance(v, (int, float)) for v in locale.values())
            odict = OrderedDict()
            for k, v in locale.items():
                key = k.replace("-", "_")
                odict[key] = v
            locales = list(odict.keys())
            self._weights = list(odict.values())

        else:
            locales = [DEFAULT_LOCALE]

        if len(locales) == 1:
            self._factory_map[locales[0]] = Factory.create(
                locales[0],
                providers,
                generator,
                includes,
                use_weighting=use_weighting,
                **config,
            )
        else:
            for locale in locales:
                self._factory_map[locale] = Faker(
                    locale,
                    providers,
                    generator,
                    includes,
                    use_weighting=use_weighting,
                    **config,
                )

        self._locales = locales
        self._factories = list(self._factory_map.values())

    def __dir__(self):
        attributes = set(super().__dir__())
        for factory in self.factories:
            attributes |= {attr for attr in dir(factory) if not attr.startswith("_")}
        return sorted(attributes)

    def __getitem__(self, locale: str) -> Faker:
        if locale.replace("-", "_") in self.locales and len(self.locales) == 1:
            return self
        instance = self._factory_map[locale.replace("-", "_")]
        assert isinstance(instance, Faker)  # for mypy
        return instance

    def __getattribute__(self, attr: str) -> Any:
        """
        Handles the "attribute resolution" behavior for declared members of this proxy class

        The class method `seed` cannot be called from an instance.

        :param attr: attribute name
        :return: the appropriate attribute
        """
        if attr == "seed":
            msg = "Calling `.seed()` on instances is deprecated. " "Use the class method `Faker.seed()` instead."
            raise TypeError(msg)
        else:
            return super().__getattribute__(attr)

    def __getattr__(self, attr: str) -> Any:
        """
        Handles cache access and proxying behavior

        :param attr: attribute name
        :return: the appropriate attribute
        """
        if len(self._factories) == 1:
            return getattr(self._factories[0], attr)
        elif attr in self.generator_attrs:
            msg = "Proxying calls to `%s` is not implemented in multiple locale mode." % attr
            raise NotImplementedError(msg)
        elif self.cache_pattern.match(attr):
            msg = "Cached attribute `%s` does not exist" % attr
            raise AttributeError(msg)
        else:
            factory = self._select_factory(attr)
            return getattr(factory, attr)

    def __deepcopy__(self, memodict):
        cls = self.__class__
        result = cls.__new__(cls)
        result._locales = copy.deepcopy(self._locales)
        result._factories = copy.deepcopy(self._factories)
        result._factory_map = copy.deepcopy(self._factory_map)
        result._weights = copy.deepcopy(self._weights)
        result._unique_proxy = UniqueProxy(self)
        result._unique_proxy._seen = {k: {result._unique_proxy._sentinel} for k in self._unique_proxy._seen.keys()}
        return result

    def __setstate__(self, state: Any) -> None:
        self.__dict__.update(state)

    @property
    def unique(self) -> UniqueProxy:
        return self._unique_proxy

    @property
    def optional(self) -> OptionalProxy:
        return self._optional_proxy

    def _select_factory(self, method_name: str) -> Factory:
        """
        Returns a random factory that supports the provider method

        :param method_name: Name of provider method
        :return: A factory that supports the provider method
        """

        factories, weights = self._map_provider_method(method_name)

        if len(factories) == 0:
            msg = f"No generator object has attribute {method_name!r}"
            raise AttributeError(msg)
        elif len(factories) == 1:
            return factories[0]

        if weights:
            factory = self._select_factory_distribution(factories, weights)
        else:
            factory = self._select_factory_choice(factories)
        return factory

    def _select_factory_distribution(self, factories, weights):
        return choices_distribution(factories, weights, random, length=1)[0]

    def _select_factory_choice(self, factories):
        return random.choice(factories)

    def _map_provider_method(self, method_name: str) -> tuple[list[Factory], list[float] | None]:
        """
        Creates a 2-tuple of factories and weights for the given provider method name

        The first element of the tuple contains a list of compatible factories.
        The second element of the tuple contains a list of distribution weights.

        :param method_name: Name of provider method
        :return: 2-tuple (factories, weights)
        """

        # Return cached mapping if it exists for given method
        attr = f"_cached_{method_name}_mapping"
        if hasattr(self, attr):
            return getattr(self, attr)

        # Create mapping if it does not exist
        if self._weights:
            value = [
                (factory, weight)
                for factory, weight in zip(self.factories, self._weights)
                if hasattr(factory, method_name)
            ]
            factories, weights = zip(*value)
            mapping = list(factories), list(weights)
        else:
            value = [factory for factory in self.factories if hasattr(factory, method_name)]  # type: ignore
            mapping = value, None  # type: ignore

        # Then cache and return results
        setattr(self, attr, mapping)
        return mapping

    @classmethod
    def seed(cls, seed: SeedType | None = None) -> None:
        """
        Hashables the shared `random.Random` object across all factories

        :param seed: seed value
        """
        Generator.seed(seed)

    def seed_instance(self, seed: SeedType | None = None) -> None:
        """
        Creates and seeds a new `random.Random` object for each factory

        :param seed: seed value
        """
        for factory in self._factories:
            factory.seed_instance(seed)

    def seed_locale(self, locale: str, seed: SeedType | None = None) -> None:
        """
        Creates and seeds a new `random.Random` object for the factory of the specified locale

        :param locale: locale string
        :param seed: seed value
        """
        self._factory_map[locale.replace("-", "_")].seed_instance(seed)

    @property
    def random(self) -> Random:
        """
        Proxies `random` getter calls

        In single locale mode, this will be proxied to the `random` getter
        of the only internal `Generator` object. Subclasses will have to
        implement desired behavior in multiple locale mode.
        """

        if len(self._factories) == 1:
            return self._factories[0].random
        else:
            msg = "Proxying `random` getter calls is not implemented in multiple locale mode."
            raise NotImplementedError(msg)

    @random.setter
    def random(self, value: Random) -> None:
        """
        Proxies `random` setter calls

        In single locale mode, this will be proxied to the `random` setter
        of the only internal `Generator` object. Subclasses will have to
        implement desired behavior in multiple locale mode.
        """

        if len(self._factories) == 1:
            self._factories[0].random = value
        else:
            msg = "Proxying `random` setter calls is not implemented in multiple locale mode."
            raise NotImplementedError(msg)

    @property
    def locales(self) -> list[str]:
        return list(self._locales)

    @property
    def weights(self) -> list[int | float] | None:
        return self._weights

    @property
    def factories(self) -> list[Generator | Faker]:
        return self._factories

    def items(self) -> list[tuple[str, Generator | Faker]]:
        return list(self._factory_map.items())


class UniqueProxy:
    def __init__(self, proxy: Faker):
        self._proxy = proxy
        self._seen: dict = {}
        self._sentinel = object()

    def clear(self) -> None:
        self._seen = {}

    def __getattr__(self, name: str) -> Any:
        obj = getattr(self._proxy, name)
        if callable(obj):
            return self._wrap(name, obj)
        else:
            raise TypeError("Accessing non-functions through .unique is not supported.")

    def __getstate__(self):
        # Copy the object's state from self.__dict__ which contains
        # all our instance attributes. Always use the dict.copy()
        # method to avoid modifying the original state.
        state = self.__dict__.copy()
        return state

    def __setstate__(self, state):
        self.__dict__.update(state)

    def _wrap(self, name: str, function: Callable) -> Callable:
        @functools.wraps(function)
        def wrapper(*args, **kwargs):
            key = (name, args, tuple(sorted(kwargs.items())))

            generated = self._seen.setdefault(key, {self._sentinel})

            # With use of a sentinel value rather than None, we leave
            # None open as a valid return value.
            retval = self._sentinel

            for i in range(_UNIQUE_ATTEMPTS):
                if retval not in generated:
                    break
                retval = function(*args, **kwargs)
            else:
                raise UniquenessException(f"Got duplicated values after {_UNIQUE_ATTEMPTS:,} iterations.")

            generated.add(retval)

            return retval

        return wrapper


class OptionalProxy:
    """
    Return either a fake value or None, with a customizable probability.
    """

    def __init__(self, proxy: Faker):
        self._proxy = proxy

    def __getattr__(self, name: str) -> Any:
        obj = getattr(self._proxy, name)
        if callable(obj):
            return self._wrap(name, obj)
        else:
            raise TypeError("Accessing non-functions through .optional is not supported.")

    def __getstate__(self):
        # Copy the object's state from self.__dict__ which contains
        # all our instance attributes. Always use the dict.copy()
        # method to avoid modifying the original state.
        state = self.__dict__.copy()
        return state

    def __setstate__(self, state):
        self.__dict__.update(state)

    def _wrap(self, name: str, function: Callable[..., RetType]) -> Callable[..., RetType | None]:
        @functools.wraps(function)
        def wrapper(*args: Any, prob: float = 0.5, **kwargs: Any) -> RetType | None:
            if not 0 < prob <= 1.0:
                raise ValueError("prob must be between 0 and 1")
            return function(*args, **kwargs) if self._proxy.boolean(chance_of_getting_true=int(prob * 100)) else None

        return wrapper



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    DAY_NAMES = {
        "0": "neděle",
        "1": "pondělí",
        "2": "úterý",
        "3": "středa",
        "4": "čtvrtek",
        "5": "pátek",
        "6": "sobota",
    }

    MONTH_NAMES = {
        "01": "leden",
        "02": "únor",
        "03": "březen",
        "04": "duben",
        "05": "květen",
        "06": "červen",
        "07": "červenec",
        "08": "srpen",
        "09": "září",
        "10": "říjen",
        "11": "listopad",
        "12": "prosinec",
    }

    def day_of_week(self):
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        return self.MONTH_NAMES[month]



from datetime import date, datetime, timedelta

from faker.providers.person.en_US import Provider as EnUSPersonProvider

female_first_name = list(EnUSPersonProvider.first_names_female.keys())
male_first_name = list(EnUSPersonProvider.first_names_male.keys())
nb_first_name = list(EnUSPersonProvider.first_names_nonbinary.keys())
last_name = list(EnUSPersonProvider.last_names.keys())


class TestPassportProvider:
    def testDOB(self, faker, num_samples=100):
        for _ in range(num_samples):
            dob = faker.passport_dob()
            age = (date.today() - dob).days // 365
            assert isinstance(dob, date)
            assert age <= 115
            assert age >= 0

    def testOwnerNames(self, faker, num_samples=100):
        for _ in range(num_samples):
            given_name, surname = faker.passport_owner("F")
            assert given_name in female_first_name
            assert surname in last_name

            given_name, surname = faker.passport_owner("M")
            assert given_name in male_first_name
            assert surname in last_name

            given_name, surname = faker.passport_owner("X")
            assert given_name in nb_first_name
            assert surname in last_name

    def testNumFormat(self, faker, num_samples=100):
        for _ in range(num_samples):
            pass_number = faker.passport_number()
            if pass_number[0].isalpha():
                assert pass_number[1:].isdigit()
            else:
                assert pass_number.isdigit()
            assert len(pass_number) == 9


class TestEnUS:
    def testDates(self, faker, num_samples=20):
        age4 = date.today() - timedelta(days=4 * 365)
        age12 = date.today() - timedelta(days=12 * 365)
        age17 = date.today() - timedelta(days=17 * 365)
        age23 = date.today() - timedelta(days=23 * 365)
        age30 = date.today() - timedelta(days=30 * 365)

        birthdays = [(age4, 4), (age12, 12), (age17, 17), (age23, 23), (age30, 30)]
        for _ in range(num_samples):
            for birthday in birthdays:
                birth_date_f, issue_date_f, expiry_date_f = faker.passport_dates(birthday[0])
                birth_date = datetime.strptime(birth_date_f, "%d %b %Y").date()
                issue_date = datetime.strptime(issue_date_f, "%d %b %Y").date()
                expiry_date = datetime.strptime(expiry_date_f, "%d %b %Y").date()

                if birthday[1] < 21:
                    assert (expiry_date - issue_date).days // 365 == 5
                else:
                    assert (expiry_date - issue_date).days // 365 == 10

                assert expiry_date > issue_date
                assert birth_date < issue_date

    def testGender(self, faker, num_samples=100):
        for _ in range(num_samples):
            assert faker.passport_gender(7899) in ["M", "F", "X"]

    def testfull(self, faker, num_samples=100):
        for _ in range(num_samples):
            pass_data = faker.passport_full().split("\n")
            assert pass_data[1] in last_name
            assert pass_data[2] in ["M", "F", "X"]
            if pass_data[2] == "M":
                assert pass_data[0] in male_first_name
            elif pass_data[2] == "F":
                assert pass_data[0] in female_first_name
            elif pass_data[2] == "X":
                assert pass_data[0] in nb_first_name
